-- CREACI DE TAULES SENSE REFERENCIAR --

-- PAS
CREATE TABLE Pais (
  idPais CHAR(2) CONSTRAINT PK_idPais PRIMARY KEY,
  nomPais VARCHAR2(50) CONSTRAINT NN_nomPais NOT NULL
);

-- LOCALITAT 
CREATE TABLE Localitat (
  idLocalitat INTEGER CONSTRAINT NN_L_idLocalitat NOT NULL,
  idPais CHAR(2) CONSTRAINT NN_L_idPais NOT NULL,
  nomLocalitat VARCHAR2(120) CONSTRAINT NN_L_nomLocalitat NOT NULL,
  CONSTRAINT PK_Localitat PRIMARY KEY (idPais, idLocalitat)
);

-- ADREA

CREATE TABLE Adreca (
  idAdreca INTEGER CONSTRAINT NN_A_idAdreca NOT NULL,
  idLocalitat INTEGER CONSTRAINT NN_A_idLocalitat NOT NULL,
  idPais CHAR(2) CONSTRAINT NN_A_idPais NOT NULL,
  codiPostal VARCHAR2(5) CONSTRAINT NN_A_codiPostal NOT NULL,
  carrer VARCHAR2(255) CONSTRAINT NN_A_carrer NOT NULL, 
  num INTEGER CONSTRAINT NN_A_num NOT NULL,
  escala CHAR(1),
  pis INTEGER CONSTRAINT NN_A_pis NOT NULL,
  porta INTEGER CONSTRAINT NN_A_porta NOT NULL,
  CONSTRAINT PK_Adreca PRIMARY KEY (idPais, idLocalitat, idAdreca)
);

-- CENTRAL DE PRODUCCIO
CREATE TABLE CentralProduccio (
  codiCentral INTEGER CONSTRAINT PK_codiCentral PRIMARY KEY,
  idPais CHAR(2) CONSTRAINT NN_CP_idPais NOT NULL,
  idLocalitat INTEGER CONSTRAINT NN_cP_idLocalitat NOT NULL,
  idAdreca INTEGER CONSTRAINT NN_CP_idAdreca NOT NULL,
  energiaMax NUMBER(10,2) CONSTRAINT NN_CP_energia NOT NULL,
  dataInspeccio TIMESTAMP CONSTRAINT NN_dataInspeccio NOT NULL,
  estat CHAR(1) CONSTRAINT ck_cp_estat CHECK 
        (estat IN ('A','B')) CONSTRAINT NN_cp_estat NOT NULL,
  dataEstat TIMESTAMP CONSTRAINT NN_cp_dataEstat NOT NULL
);

-- CENTRAL DE PRODUCCIO ENERGIA SOLAR
CREATE TABLE Solar (
  codiCentral INTEGER CONSTRAINT PK_S_codiCentral PRIMARY KEY,
  panells INTEGER CONSTRAINT NN_panells NOT NULL
);

-- CENTRAL PRODUCCIO ENERGIA TERMICA
CREATE TABLE Termica (
  codiCentral INTEGER CONSTRAINT PK_T_codiCentral PRIMARY KEY,
  emissions NUMBER(10,2) CONSTRAINT NN_emissions NOT NULL
);

-- CENTRAL PRODUCCIO ENERGIA ELICA
CREATE TABLE Eolica (
  codiCentral INTEGER CONSTRAINT PK_E_codiCentral PRIMARY KEY,
  molins INTEGER CONSTRAINT NN_molins NOT NULL
);

-- CENTRAL PRODUCCIO ENERGIA HIDROELECTRICA
CREATE TABLE Hidroelectrica (
  codiCentral INTEGER CONSTRAINT PK_H_codiCentral PRIMARY KEY,
  turbines INTEGER CONSTRAINT NN_turbines NOT NULL
);

-- CENTRAL PRODUCCIO ENERGIA NUCLEAR
CREATE TABLE Nuclear (
  codiCentral INTEGER CONSTRAINT PK_N_codiCentral PRIMARY KEY,
  energiaMin NUMBER(10,2) CONSTRAINT NN_energiaMin NOT NULL ,
  rebuig NUMBER(10,2) CONSTRAINT NN_rebuig NOT NULL 
);

-- PRODUCCIO
CREATE TABLE Produccio (
  dataLectura TIMESTAMP,
  codiCentral INTEGER,
  ProduccioMes NUMBER(10,2) CONSTRAINT NN_produccioMes NOT NULL,
  PRIMARY KEY (dataLectura, codiCentral)
);

-- CENTRALETA DISTRIBUCIO D'ENERGIA
CREATE TABLE CentraletaDistribucio (
  codiCentraleta INTEGER CONSTRAINT NN_cd_codiCentraleta NOT NULL,
  idPais CHAR(2) CONSTRAINT NN_CD_idPais NOT NULL,
  idLocalitat INTEGER CONSTRAINT NN_CD_idLocalitat NOT NULL,
  idAdreca INTEGER CONSTRAINT NN_CD_idAdreca NOT NULL,
  capacitatCentraleta NUMBER(10,2) CONSTRAINT NN_cd_capacitatCentraleta NOT NULL,
  estat CHAR(1) CONSTRAINT ck_cd_estat CHECK 
        (estat IN ('A','B')) CONSTRAINT NN_cd_estat NOT NULL,
  dataEstat TIMESTAMP CONSTRAINT NN_cd_dataEstat NOT NULL,
  CONSTRAINT PK_CD PRIMARY KEY (codiCentraleta, idPais, idLocalitat, idAdreca)
);

-- LINIES DE COMUNICACIO
CREATE TABLE liniesComunicacio (
  idLinia INTEGER CONSTRAINT PK_idLinia PRIMARY KEY,
  capacitatLinia NUMBER(10,2) CONSTRAINT NN_capacitatLinia NOT NULL,
  estat CHAR(1) CONSTRAINT ck_LC_estat CHECK 
        (estat IN ('A','B')) CONSTRAINT NN_LC_estat NOT NULL,
  dataEstat TIMESTAMP CONSTRAINT NN_LC_dataEstat NOT NULL
);

-- CLIENT
CREATE TABLE Client (
  DNI VARCHAR(10) CONSTRAINT PK_DNI PRIMARY KEY,
  nomClient VARCHAR2(255) CONSTRAINT NN_nomClient NOT NULL,
  idPais CHAR(2) CONSTRAINT NN_cl_idPais NOT NULL,
  idLocalitat INTEGER CONSTRAINT NN_cl_idLocalitat NOT NULL,
  idAdreca INTEGER CONSTRAINT NN_cl_idAdreca NOT NULL,
  estat CHAR(1) CONSTRAINT ck_cl_estat CHECK 
        (estat IN ('A','B')) CONSTRAINT NN_cl_estat NOT NULL,
  dataEstat TIMESTAMP CONSTRAINT NN_cl_dataEstat NOT NULL
);

-- COMPTADOR
CREATE TABLE Comptador (
  codiContracte INTEGER CONSTRAINT PK_codiContracte PRIMARY KEY,
  potenciaContractada NUMBER(10,2) CONSTRAINT NN_co_potenciaContractada NOT NULL,
  modelComptador VARCHAR2(255) CONSTRAINT NN_co_modelComptador NOT NULL,
  empresaFabricacio VARCHAR2(255) CONSTRAINT NN_co_empresaFabricacio NOT NULL,
  dataFabricacio TIMESTAMP CONSTRAINT NN_co_dataFabricacio NOT NULL,
  estat CHAR(1) CONSTRAINT ck_co_estat CHECK 
        (estat IN ('A','B')) CONSTRAINT NN_co_estat NOT NULL,
  dataEstat TIMESTAMP CONSTRAINT NN_co_dataEstat NOT NULL,
  idPais CHAR(2) CONSTRAINT NN_co_idPais NOT NULL,
  idLocalitat INTEGER CONSTRAINT NN_co_idLocalitat NOT NULL,
  idAdreca INTEGER CONSTRAINT NN_co_idAdreca NOT NULL,
  DNI VARCHAR(10) CONSTRAINT NN_co_DNI NOT NULL
);

-- CONSUM
CREATE TABLE Consum (
  dataLectura TIMESTAMP,
  codiContracte INTEGER,
  consum NUMBER(10,2) CONSTRAINT NN_consum NOT NULL,
  tipusLectura CHAR(1) CONSTRAINT ck_co_tipus CHECK 
        (tipusLectura IN ('P','T')) CONSTRAINT NN_co_tipus NOT NULL,
  CONSTRAINT PK_CONSUM PRIMARY KEY (dataLectura, codiContracte)
);

-- INTERRELACIO N:M:1 (COMPTADOR, CENTRALETA, LINIA)
CREATE TABLE Connecta(
  codiContracte INTEGER,
  codiCentraleta INTEGER,
  idPais CHAR(2),
  idLocalitat INTEGER,
  idAdreca INTEGER,
  idLinia INTEGER,
  CONSTRAINT PK_CONECTA PRIMARY KEY (codiContracte, idLinia)
);

-- INTERRELACI N:1:1 (CENTRALETA, CENTRAL, LINIA COMUNICACIO)
CREATE TABLE Alimenta (
  codiCentraleta INTEGER,
  idPais CHAR(2),
  idLocalitat INTEGER,
  idAdreca INTEGER,
  codiCentral INTEGER,
  idLinia INTEGER,
  CONSTRAINT PK_ALIMENTA PRIMARY KEY (codiCentraleta, idPais, idLocalitat, idAdreca, codiCentral)
);

-- ALTERS QUE REFERENCIEN A CLAUS FORANES
ALTER TABLE Pais
ADD CONSTRAINT UN_pais_nom UNIQUE (nomPais);

ALTER TABLE Localitat
ADD CONSTRAINT FK1_Localitat_Pais FOREIGN KEY (idPais) REFERENCES Pais(idPais);

ALTER TABLE Adreca
ADD CONSTRAINT FK1_Adreca_Localitat FOREIGN KEY (idPais,idLocalitat) REFERENCES Localitat(idPais,idLocalitat);

ALTER TABLE CentralProduccio
ADD CONSTRAINT FK1_CentralProduccio_Adreca FOREIGN KEY (idPais, idLocalitat,idAdreca) REFERENCES Adreca(idPais, idLocalitat,idAdreca);

ALTER TABLE Consum
ADD CONSTRAINT FK1_Consum_Comptador FOREIGN KEY (codiContracte) REFERENCES Comptador(codiContracte);

ALTER TABLE Produccio
ADD CONSTRAINT FK1_Produccio_CentralProduccio FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

ALTER TABLE Client
ADD CONSTRAINT FK1_Client_Adreca FOREIGN KEY (idPais, idLocalitat,idAdreca) REFERENCES Adreca(idPais, idLocalitat,idAdreca);

ALTER TABLE Comptador
ADD CONSTRAINT FK1_Comptador_Adreca FOREIGN KEY (idPais, idLocalitat,idAdreca) REFERENCES Adreca(idPais, idLocalitat,idAdreca);

ALTER TABLE CentraletaDistribucio
ADD CONSTRAINT FK1_Centraleta_Adreca FOREIGN KEY (idPais, idLocalitat,idAdreca) REFERENCES Adreca(idPais, idLocalitat,idAdreca);

ALTER TABLE Comptador
ADD CONSTRAINT FK1_Comptador_Client FOREIGN KEY (DNI) REFERENCES Client(DNI);

ALTER TABLE Nuclear
ADD CONSTRAINT FK1_Nuclear_codiCentral FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

ALTER TABLE Termica
ADD CONSTRAINT FK1_Termica_codiCentral FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

ALTER TABLE Hidroelectrica
ADD CONSTRAINT FK1_Hidro_codiCentral FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

ALTER TABLE Eolica
ADD CONSTRAINT FK1_Eolica_codiCentral FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

ALTER TABLE Solar
ADD CONSTRAINT FK1_Solar_codiCentral FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

ALTER TABLE  Connecta
ADD CONSTRAINT FK1_Connecta_codiContracte FOREIGN KEY (codiContracte) REFERENCES Comptador(codiContracte);

ALTER TABLE Connecta
ADD CONSTRAINT FK2_Connecta_codiCentraleta FOREIGN KEY (codiCentraleta,idPais,idLocalitat,idAdreca) REFERENCES CentraletaDistribucio(codiCentraleta,idPais,idLocalitat,idAdreca);

ALTER TABLE Connecta
ADD CONSTRAINT FK3_Connceta_idLinia FOREIGN KEY (idLinia) REFERENCES liniesComunicacio(idLinia);

ALTER TABLE Alimenta 
ADD CONSTRAINT FK1_Alimenta_codiCentral FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

ALTER TABLE Alimenta 
ADD CONSTRAINT FK2_Alimenta_codiCentraleta FOREIGN KEY (codiCentraleta, idPais,idLocalitat,idAdreca) REFERENCES CentraletaDistribucio(codiCentraleta, idPais,idLocalitat,idAdreca);

ALTER TABLE Alimenta 
ADD CONSTRAINT FK3_Alimenta_idLinia FOREIGN KEY (idLinia) REFERENCES liniesComunicacio(idLinia);

-- CREACIO TAULES AUXILIARS
--LOGS 
CREATE TABLE Logs (
  idLog INTEGER CONSTRAINT PK_idLog PRIMARY KEY,
  nomProcediment VARCHAR2(80) CONSTRAINT NN_nomProcedicment NOT NULL,
  paramEntrada VARCHAR2(4000) CONSTRAINT NN_paramEntrada NOT NULL,
  paramSortida VARCHAR2(4000) CONSTRAINT NN_paramSortida NOT NULL, 
  resultat VARCHAR2(4000) CONSTRAINT NN_resultat NOT NULL,
  usuariBD VARCHAR2(50) CONSTRAINT NN_usuariBD NOT NULL,
  data_hora TIMESTAMP CONSTRAINT NN_datahora NOT NULL
);

-- ESTADISTICA 1: CONSUM DELS COMPTADORS QUE DEPENEN D'UNA CENTRAL
CREATE TABLE E1 (
  codiCentral INTEGER CONSTRAINT PK_E1_codiContracte PRIMARY KEY,
  consum NUMBER(10,2) CONSTRAINT NN_E1_consum NOT NULL
);

ALTER TABLE E1
ADD CONSTRAINT FK1_E1_codiCentral FOREIGN KEY (codiCentral) REFERENCES CentralProduccio(codiCentral);

-- ESTADISTICA 2: Energia mitjana consumida en els comptadors d'una linia
CREATE TABLE E2 (
   ao INTEGER,
   idLinia INTEGER,
   energiaConsumida NUMBER(20,2),
   PRIMARY KEY (ao, idLinia)
);

ALTER TABLE E2
ADD CONSTRAINT FK1_E2_idLinia FOREIGN KEY (idLinia) REFERENCES liniesComunicacio(idLinia);

-- ESTADISTICA 3: LINIA MES CARREGADA
CREATE TABLE E3 (
   idLinia INTEGER
);

-- ESTADISTICA 4: PERCENTATGE DE  LNIES QUE SUPEREN EL 50% DE L'ENERGIA CONSUMIDA EN UN ANY
CREATE TABLE E4 (
   ao INTEGER CONSTRAINT PK_E4_any PRIMARY KEY,
   percentatge NUMBER(5,2)
);

-- ESTADISTICA 5: NOMBRE DE CENTRALS DE PRODUCCIO QUE GENEREN MENYS DEL 30% DE LA PRODUCCIO EN UN ANY CONCRET
CREATE TABLE E5 (
   ao INTEGER CONSTRAINT PK_E5_any PRIMARY KEY,
   centrals INTEGER
);

-- ESTADISTICA 6: PROPORCIONA EL TOP-10 DE COMPTADORS AMB MS CONSUM HISTRIC
CREATE TABLE E6 (
   codiContracte INTEGER CONSTRAINT PK_E6_codiContracte PRIMARY KEY,
   consumHistoric NUMBER(20,2)
);

-- ESTADISTICA 7: CONSUM MIG DE TOTS ELS CLIENTS
CREATE TABLE E7 (
    consumMig NUMBER(20,2)
);
