-- GESTIO DE CONNEXIONS ENTRE CENTRALS DE PRODUCCI I CENTRALETES DE DISTRIBUCI --

------------------------------------------------------
-- ALIMENTA - insert
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca, codiCentral, idLinia
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Alimenta_insert (
	PcodiCentraleta IN Alimenta.codiCentraleta%TYPE,
	PidPais IN Alimenta.idPais%TYPE,
	PidLocalitat IN Alimenta.idLocalitat%TYPE,
	PidAdreca IN Alimenta.idAdreca%TYPE,
	PcodiCentral IN Alimenta.codiCentral%TYPE,
	PidLinia IN Alimenta.idLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Alimenta_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatLinia CHAR(1);
	estatCentraleta CHAR(1);
	estatCentral CHAR(1);

	carregaLinia NUMBER(10,2);
	carrega NUMBER(10,2);
	capacitat NUMBER(10,2);

	noCodiCentraleta EXCEPTION;
	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noCodiCentral EXCEPTION;
	noidLinia EXCEPTION;
	noTrobatCentraleta EXCEPTION;
	noTrobatCentral EXCEPTION;
	noTrobatLinia EXCEPTION;
	CapacitatMaxima EXCEPTION;
	CentralNoActiva EXCEPTION;
	CentraletaNoActiva EXCEPTION;
	LiniaNoActiva EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca ||
			'> <' || PcodiCentral || '> <' || Pidlinia || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiCentraleta IS NULL OR PcodiCentraleta ='') THEN
		RAISE noCodiCentraleta;
	END IF;

	IF (PidPais IS NULL OR PidPais ='') THEN
		RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat ='') THEN
		RAISE noLoc;
	END IF;

	IF (PidAdreca IS NULL OR PidAdreca ='') THEN
		RAISE noAdreca;
	END IF;

	IF (PcodiCentral IS NULL OR PcodiCentral ='') THEN
		RAISE noCodiCentral;
	END IF;

	IF (PidLinia IS NULL OR PidLinia ='') THEN
		RAISE noidLinia;
	END IF;

	-- Comprovaci existencia de Central, Centraleta i Linia

	BEGIN 
		SELECT estat INTO estatCentral FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION		
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentral;
	END;

	BEGIN 
		SELECT estat INTO estatCentraleta FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentraleta;		
	END;

	BEGIN 
		SELECT estat INTO estatLinia FROM LiniesComunicacio
		WHERE idLinia = PidLinia;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatLinia;		
	END;
	
	IF (estatLinia <> 'A') THEN
		RAISE LiniaNoActiva;
  	END IF;
	IF (estatCentral <> 'A') THEN
		RAISE CentralNoActiva;
	END IF;
	IF (estatCentraleta <> 'A') THEN
		RAISE CentraletaNoActiva;
	END IF;

	-- Comprovar que la capacitat mxima de totes les centraletes connectades a la Linia no supera la capacitat de la linia
	-- S'inclou en el clcul la crrega de les centraletes donades de baixa per de les que no s'ha desmantellat les connexions amb les centrals.
	-- Pot ser que la centraleta estigui de baixa temporalment per avaria.
	SELECT SUM(CentraletaDistribucio.capacitatCentraleta) INTO carregaLinia
		FROM CentraletaDistribucio INNER JOIN Alimenta ON (
			CentraletaDistribucio.codiCentraleta=Alimenta.codiCentraleta AND
			CentraletaDistribucio.idPais=Alimenta.idPais AND
			CentraletaDistribucio.idLocalitat=Alimenta.idLocalitat AND
			CentraletaDistribucio.idAdreca=Alimenta.idAdreca)
		WHERE Alimenta.idLinia=PidLinia; 

  IF (carregaLinia IS NULL) THEN 
    carregaLinia := 0.0;
  END IF;
  
	SELECT capacitatCentraleta INTO carrega FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);

	SELECT capacitatLinia INTO capacitat FROM LiniesComunicacio
		WHERE idLinia=PidLinia;

	carregaLinia:=carregaLinia + carrega;

	-- Si no es supera la capacitat maxima s'inserta a la taula
	IF (carregaLinia <= capacitat) THEN
		INSERT INTO Alimenta(codiCentraleta,idPais,idLocalitat,idAdreca,codiCentral,idLinia) 
			VALUES (PcodiCentraleta,PidPais,PidLocalitat,PidAdreca,PcodiCentral,PidLinia);
	ELSE
		RAISE CapacitatMaxima;
	END IF;	

        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <CodiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <CodiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLinia THEN
				resultat := 'ERROR: <idLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta  THEN
				resultat := 'ERROR: No s ha trobat la centraleta de distribucio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral  THEN
				resultat := 'ERROR: No s ha trobat la central de produccio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLinia  THEN
				resultat := 'ERROR: No s ha trobat la linia de comunicacio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CapacitatMaxima  THEN
				resultat := 'ERROR: La linia esta al maxim de capacitat. No s hi poden connectar mes centraletes';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentraletaNoActiva  THEN
				resultat := 'ERROR: La Centraleta no est activa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentralNoActiva  THEN
				resultat := 'ERROR: La Central no est activa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN LiniaNoActiva  THEN
				resultat := 'ERROR: La linia no est activa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: La Centraleta i la Central ja estan connectades amb aquesta linia';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Alimenta_insert;
/

------------------------------------------------------
-- ALIMENTA - delete d'una connexio concreta
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca, codiCentral, idLinia
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Alimenta_delete (
	PcodiCentraleta IN Alimenta.codiCentraleta%TYPE,
	PidPais IN Alimenta.idPais%TYPE,
	PidLocalitat IN Alimenta.idLocalitat%TYPE,
	PidAdreca IN Alimenta.idAdreca%TYPE,
	PcodiCentral IN Alimenta.codiCentral%TYPE,
	PidLinia IN Alimenta.idLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Alimenta_delete';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatLinia CHAR(1);
	estatCentraleta CHAR(1);
	estatCentral CHAR(1);
  
	noCodiCentraleta EXCEPTION;
	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noCodiCentral EXCEPTION;
	noidLinia EXCEPTION;
	noTrobatCentraleta EXCEPTION;
	noTrobatCentral EXCEPTION;
	noTrobatLinia EXCEPTION;
	noAlimenta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca ||
			'> <' || PcodiCentral || '> <' || Pidlinia || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiCentraleta IS NULL OR PcodiCentraleta ='') THEN
		RAISE noCodiCentraleta;
	END IF;

	IF (PidPais IS NULL OR PidPais ='') THEN
		RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat ='') THEN
		RAISE noLoc;
	END IF;

	IF (PidAdreca IS NULL OR PidAdreca ='') THEN
		RAISE noAdreca;
	END IF;

	IF (PcodiCentral IS NULL OR PcodiCentral ='') THEN
		RAISE noCodiCentral;
	END IF;

	IF (PidLinia IS NULL OR PidLinia ='') THEN
		RAISE noidLinia;
	END IF;

	-- Comprovaci existencia de Central, Centraleta i Linia

	BEGIN 
		SELECT estat INTO estatCentral FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION		
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentral;
	END;

	BEGIN 
		SELECT estat INTO estatCentraleta FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentraleta;		
	END;

	BEGIN 
		SELECT estat INTO estatLinia FROM LiniesComunicacio
		WHERE idLinia = PidLinia;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatLinia;		
	END;
	-- S'esborren noms si la connexi no ha tingut consums (=si la central no ha tingut consums)
	BEGIN 
		DELETE FROM Alimenta al
			WHERE (al.codiCentraleta=PcodiCentraleta AND al.idPais=PidPais AND al.idLocalitat=PidLocalitat 
			AND al.idAdreca=PidAdreca AND al.codiCentral=PcodiCentral AND al.idLinia=PidLinia 
      AND al.codiCentral NOT IN  (select cp.codiCentral
                              from consum cs,centralProduccio cp
                              where cs.codiContracte IN (
                                    select distinct cn.codiContracte
                                    from connecta cn
                                    inner join alimenta al
                                    on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                    and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                    where al.codiCentral =cp.codiCentral)
                                    having sum(cs.consum)>0
                                    group by cp.CodiCentral));
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noAlimenta;
	END;
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <CodiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <CodiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLinia THEN
				resultat := 'ERROR: <idLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta  THEN
				resultat := 'ERROR: No s ha trobat la centraleta de distribucio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral  THEN
				resultat := 'ERROR: No s ha trobat la central de produccio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLinia  THEN
				resultat := 'ERROR: No s ha trobat la linia de comunicacio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoAlimenta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o hi ha hagut consums a travs d aquesta connexi';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Alimenta_delete;
/

------------------------------------------------------
-- ALIMENTA - delete totes les connexions a Central d'una centraleta donada
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Alimenta_delCentraleta (
	PcodiCentraleta IN Alimenta.codiCentraleta%TYPE,
	PidPais IN Alimenta.idPais%TYPE,
	PidLocalitat IN Alimenta.idLocalitat%TYPE,
	PidAdreca IN Alimenta.idAdreca%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Alimenta_delCentraleta';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatCentraleta CHAR(1);
  
	noCodiCentraleta EXCEPTION;
	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noTrobatCentraleta EXCEPTION;
	noAlimenta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiCentraleta IS NULL OR PcodiCentraleta ='') THEN
		RAISE noCodiCentraleta;
	END IF;

	IF (PidPais IS NULL OR PidPais ='') THEN
		RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat ='') THEN
		RAISE noLoc;
	END IF;

	IF (PidAdreca IS NULL OR PidAdreca ='') THEN
		RAISE noAdreca;
	END IF;

	-- Comprovaci existencia de Centraleta 
	BEGIN 
		SELECT estat INTO estatCentraleta FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentraleta;		
	END;

	BEGIN 
		DELETE FROM Alimenta al
			WHERE (al.codiCentraleta=PcodiCentraleta AND al.idPais=PidPais AND al.idLocalitat=PidLocalitat 
			AND al.idAdreca=PidAdreca AND al.codiCentral NOT IN  (select cp.codiCentral
                              from consum cs,centralProduccio cp
                              where cs.codiContracte IN (
                                    select distinct cn.codiContracte
                                    from connecta cn
                                    inner join alimenta al
                                    on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                    and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                    where al.codiCentral =cp.codiCentral)
                                    having sum(cs.consum)>0
                                    group by cp.CodiCentral));
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noAlimenta;
	END;
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <CodiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta  THEN
				resultat := 'ERROR: No s ha trobat la centraleta de distribucio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoAlimenta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o hi ha hagut consums a travs d aquesta connexi';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Alimenta_delCentraleta;
/

------------------------------------------------------
-- ALIMENTA - delete totes les connexions a centraleta d'una Central donada
-- Parmetres d'entrada: codiCentral
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Alimenta_delCentral (
	PcodiCentral IN Alimenta.codiCentral%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Alimenta_delCentral';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatCentral CHAR(1);
  
	noCodiCentral EXCEPTION;
	noTrobatCentral EXCEPTION;

	noAlimenta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiCentral IS NULL OR PcodiCentral ='') THEN
		RAISE noCodiCentral;
	END IF;

	-- Comprovaci existencia de Central

	BEGIN 
		SELECT estat INTO estatCentral FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION		
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentral;
	END;

	BEGIN 
		DELETE FROM Alimenta al
			WHERE (al.codiCentral=PcodiCentral AND al.codiCentral NOT IN  (select cp.codiCentral
                              from consum cs,centralProduccio cp
                              where cs.codiContracte IN (
                                    select distinct cn.codiContracte
                                    from connecta cn
                                    inner join alimenta al
                                    on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                    and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                    where al.codiCentral =cp.codiCentral)
                                    having sum(cs.consum)>0
                                    group by cp.CodiCentral));
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noAlimenta;
	END;
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <CodiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: No s ha trobat la central de produccio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoAlimenta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o la central ha tingut consums';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Alimenta_delCentral;
/

------------------------------------------------------
-- ALIMENTA - delete de totes les connexions d'una linia concreta
-- Parmetres d'entrada: idLinia
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Alimenta_delLinia (
	PidLinia IN Alimenta.idLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Alimenta_delLinia';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatLinia CHAR(1);
	noidLinia EXCEPTION;
	noTrobatLinia EXCEPTION;
	noAlimenta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || Pidlinia || '>';

	-- Comprovaci parmetres entrada
	IF (PidLinia IS NULL OR PidLinia ='') THEN
		RAISE noidLinia;
	END IF;

	-- Comprovaci existencia de Central, Centraleta i Linia
	BEGIN 
		SELECT estat INTO estatLinia FROM LiniesComunicacio
		WHERE idLinia = PidLinia;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatLinia;		
	END;
	
	BEGIN 
		DELETE FROM Alimenta al
			WHERE (al.idLinia=PidLinia AND al.codiCentral NOT IN  (select cp.codiCentral
                              from consum cs,centralProduccio cp
                              where cs.codiContracte IN (
                                    select distinct cn.codiContracte
                                    from connecta cn
                                    inner join alimenta al
                                    on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                    and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                    where al.codiCentral =cp.codiCentral)
                                    having sum(cs.consum)>0
                                    group by cp.CodiCentral));
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noAlimenta;
	END;
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noidLinia THEN
				resultat := 'ERROR: <idLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLinia  THEN
				resultat := 'ERROR: No s ha trobat la linia de comunicacio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoAlimenta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o les linies han tingut consums';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Alimenta_delLinia;
/


