-- GESTIO CENTRALETES I LINIES COMUNICACIO (ALTA-BAIXA-MODIFICACIO) --

------------------------------------------------------
-- ALIMENTA - insert
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca, codiCentral, idLinia
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Connecta_insert (
	PcodiContracte IN Connecta.codiContracte%TYPE,
	PcodiCentraleta IN Connecta.codiCentraleta%TYPE,
	PidPais IN Connecta.idPais%TYPE,
	PidLocalitat IN Connecta.idLocalitat%TYPE,
	PidAdreca IN Connecta.idAdreca%TYPE,
	PidLinia IN Connecta.idLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Connecta_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatLinia CHAR(1);
	estatCentraleta CHAR(1);
	estatContracte CHAR(1);

	potenciaTotal NUMBER(10,2);
	potencia NUMBER(10,2);
  potenciaNova NUMBER(10,2);
	capacitat NUMBER(10,2);
	capacitatL NUMBER(10,2);
  
	noCodiCentraleta EXCEPTION;
	noidPais EXCEPTION;
	noidLocalitat EXCEPTION;
	noidAdreca EXCEPTION;
	noCodiContracte EXCEPTION;
	noidLinia EXCEPTION;
	noTrobatCentraleta EXCEPTION;
	noTrobatContracte EXCEPTION;
	noTrobatLinia EXCEPTION;
	noCapacitat EXCEPTION;
	ComptadorNoActiu EXCEPTION;
	CentraletaNoActiva EXCEPTION;
	LiniaNoActiva EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiContracte || '> <' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca ||
			'> <' || Pidlinia || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiCentraleta IS NULL OR PcodiCentraleta ='') THEN
		RAISE noCodiCentraleta;
	END IF;

	IF (PidPais IS NULL OR PidPais ='') THEN
		RAISE noidPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat ='') THEN
		RAISE noidLocalitat;
	END IF;

	IF (PidAdreca IS NULL OR PidAdreca ='') THEN
		RAISE noidAdreca;
	END IF;

	IF (PcodiContracte IS NULL OR PcodiContracte ='') THEN
		RAISE noCodiContracte;
	END IF;

	IF (PidLinia IS NULL OR PidLinia ='') THEN
		RAISE noidLinia;
	END IF;

	-- Comprovaci existencia de Comptador, Centraleta i Linia

	BEGIN 
		SELECT estat INTO estatContracte FROM Comptador
		WHERE codiContracte = PcodiContracte;
	EXCEPTION		
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatContracte;
	END;

	BEGIN 
		SELECT estat INTO estatCentraleta FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentraleta;		
	END;

	BEGIN 
		SELECT estat INTO estatLinia FROM LiniesComunicacio
		WHERE idLinia = PidLinia;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatLinia;		
	END;
	
	IF (estatLinia <> 'A') THEN
		RAISE LiniaNoActiva;
  	END IF;
	IF (estatContracte <> 'A') THEN
		RAISE ComptadorNoActiu;
	END IF;
	IF (estatCentraleta <> 'A') THEN
		RAISE CentraletaNoActiva;
	END IF;

	-- Comprovar que la potencia de tots els contractes connectats a la centraleta no supera la capacitat de la centraleta
	-- S'inclou en el clcul la potncia dels comptadors donats de baixa per de les que no s'ha desmantellat les connexions amb les centraletes.
	-- Pot ser que la centraleta estigui de baixa temporalment per avaria.
	SELECT SUM(Comptador.PotenciaContractada) INTO potenciaTotal
		FROM Comptador INNER JOIN Connecta cn ON (
			Comptador.codiContracte=cn.codiContracte)
		WHERE cn.codiCentraleta=PcodiCentraleta and cn.idPais=PidPais and cn.idLocalitat=PidLocalitat and cn.idAdreca=PidAdreca; 
  IF (potenciaTotal) IS NULL THEN
    potenciaTotal:=0.0;
  END IF;
  
  SELECT potenciaContractada INTO potenciaNova
    FROM Comptador 
  WHERE codiContracte=PcodiContracte;
  
  potenciaTotal:=potenciaTotal+potenciaNova;
  
	SELECT capacitatCentraleta INTO capacitat FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);

	SELECT capacitatLinia INTO capacitatL FROM LiniesComunicacio
		WHERE idLinia=PidLinia;
  
	-- Si no es supera la capacitat maxima s'inserta a la taula
	IF (potenciaTotal <= capacitatL AND capacitatL <=capacitat) THEN
		INSERT INTO Connecta(codiContracte,codiCentraleta,idPais,idLocalitat,idAdreca,idLinia) 
			VALUES (PcodiContracte,PcodiCentraleta,PidPais,PidLocalitat,PidAdreca,PidLinia);
	ELSE
		RAISE noCapacitat;
	END IF;	

        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <CodiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLocalitat THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCodiContracte THEN
				resultat := 'ERROR: <codiContracte> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLinia THEN
				resultat := 'ERROR: <idLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta  THEN
				resultat := 'ERROR: No s ha trobat la centraleta de distribucio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatContracte  THEN
				resultat := 'ERROR: No s ha trobat la contracte de produccio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLinia  THEN
				resultat := 'ERROR: No s ha trobat la linia de comunicacio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCapacitat  THEN
				resultat := 'ERROR: La centraleta esta al maxim de capacitat. No s hi poden connectar mes contractes';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentraletaNoActiva  THEN
				resultat := 'ERROR: La Centraleta no est activa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN ComptadorNoActiu  THEN
				resultat := 'ERROR: El contracte no est activa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN LiniaNoActiva  THEN
				resultat := 'ERROR: La linia no est activa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: La Comptadors i la Centraleta ja estan connectades amb aquesta linia';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Connecta_insert;
/

------------------------------------------------------
-- Conneta - delete d'una connexio concreta
-- Parmetres d'entrada: codiContracte,codiCentraleta, idPais, idLocalitat, idAdreca, idLinia
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Connecta_delete (
	PcodiContracte IN Connecta.codiContracte%TYPE,
	PcodiCentraleta IN Connecta.codiCentraleta%TYPE,
	PidPais IN Connecta.idPais%TYPE,
	PidLocalitat IN Connecta.idLocalitat%TYPE,
	PidAdreca IN Connecta.idAdreca%TYPE,
	PidLinia IN Connecta.idLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Connecta_delete';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatLinia CHAR(1);
	estatCentraleta CHAR(1);
	estatContracte CHAR(1);
  numConnect INTEGER;  
  
	noCodiCentraleta EXCEPTION;
	noidPais EXCEPTION;
	noidLocalitat EXCEPTION;
	noidAdreca EXCEPTION;
	noCodiContracte EXCEPTION;
	noidLinia EXCEPTION;
	noTrobatCentraleta EXCEPTION;
	noTrobatContracte EXCEPTION;
	noTrobatLinia EXCEPTION;
	noConnecta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiContracte || '> <' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca ||
			 '> <' || Pidlinia || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiCentraleta IS NULL OR PcodiCentraleta ='') THEN
		RAISE noCodiCentraleta;
	END IF;

	IF (PidPais IS NULL OR PidPais ='') THEN
		RAISE noidPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat ='') THEN
		RAISE noidLocalitat;
	END IF;

	IF (PidAdreca IS NULL OR PidAdreca ='') THEN
		RAISE noidAdreca;
	END IF;

	IF (PcodiContracte IS NULL OR PcodiContracte ='') THEN
		RAISE noCodiContracte;
	END IF;

	IF (PidLinia IS NULL OR PidLinia ='') THEN
		RAISE noidLinia;
	END IF;

	-- Comprovaci existencia de Contracte, Centraleta i Linia
	BEGIN 
		SELECT estat INTO estatContracte FROM Comptador
		WHERE codiContracte = PcodiContracte;
	EXCEPTION		
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatContracte;
	END;

	BEGIN 
		SELECT estat INTO estatCentraleta FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentraleta;		
	END;

	BEGIN 
		SELECT estat INTO estatLinia FROM LiniesComunicacio
		WHERE idLinia = PidLinia;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatLinia;		
	END;
	
-- Esborrem noms si a travs d'aquesta connexi no hi ha hagut cap mena de consum

		SELECT count(cn.codiCentraleta) INTO numConnect FROM Connecta cn
			WHERE (cn.codiCentraleta=PcodiCentraleta AND cn.idPais=PidPais AND cn.idLocalitat=PidLocalitat 
			AND cn.idAdreca=PidAdreca AND cn.codiContracte=PcodiContracte AND cn.idLinia=PidLinia AND cn.codiContracte NOT IN (select cnn.codiContracte from consum cnn));

		IF numConnect <=0 THEN
        RAISE noConnecta;
    END IF;
        
		DELETE FROM Connecta cn
			WHERE (cn.codiCentraleta=PcodiCentraleta AND cn.idPais=PidPais AND cn.idLocalitat=PidLocalitat 
			AND cn.idAdreca=PidAdreca AND cn.codiContracte=PcodiContracte AND cn.idLinia=PidLinia AND cn.codiContracte NOT IN (select cnn.codiContracte from consum cnn));
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <CodiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLocalitat THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCodiContracte THEN
				resultat := 'ERROR: <CodiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLinia THEN
				resultat := 'ERROR: <idLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta  THEN
				resultat := 'ERROR: No s ha trobat la centraleta de distribucio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatContracte  THEN
				resultat := 'ERROR: No s ha trobat la central de produccio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLinia  THEN
				resultat := 'ERROR: No s ha trobat la linia de comunicacio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoConnecta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o hi ha hagut consums a travs d aquesta connexi ';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Connecta_delete;
/

------------------------------------------------------
-- Conneta - delete les connexions d'un comptador donat
-- Parmetres d'entrada: codiContracte
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Connecta_delContracte (
	PcodiContracte IN Connecta.codiContracte%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Connecta_delContracte';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatContracte CHAR(1);
  numConnect INTEGER;
  
	noCodiContracte EXCEPTION;

	noTrobatContracte EXCEPTION;
	noConnecta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiContracte || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiContracte IS NULL OR PcodiContracte ='') THEN
		RAISE noCodiContracte;
	END IF;


	-- Comprovaci existencia de Contracte
	BEGIN 
		SELECT estat INTO estatContracte FROM Comptador
		WHERE codiContracte = PcodiContracte;
	EXCEPTION		
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatContracte;
	END;
	
  SELECT count(codiContracte) into numConnect FROM Connecta cn
			WHERE (cn.codiContracte=PcodiContracte and cn.codiContracte NOT IN (SELECT cnn.codiContracte FROM consum cnn));
    
  IF numConnect <=0 THEN
			RAISE noConnecta;
	END IF;
	
		DELETE FROM Connecta cn
			WHERE (cn.codiContracte=PcodiContracte and cn.codiContracte NOT IN (SELECT cnn.codiContracte FROM consum cnn));
	
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiContracte THEN
				resultat := 'ERROR: <CodiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatContracte  THEN
				resultat := 'ERROR: No s ha trobat la central de produccio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoConnecta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o el contracta ha fet consumit ';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Connecta_delContracte;
/

------------------------------------------------------
-- Connecta - delete les connexions d'una linia donada
-- Parmetres d'entrada: idLinia
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Connecta_delLinia (
	PidLinia IN Connecta.idLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Connecta_delLinia';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatLinia CHAR(1);
  
	noidLinia EXCEPTION;
	noTrobatLinia EXCEPTION;
	noConnecta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || Pidlinia || '>';

	-- Comprovaci parmetres entrada
	IF (PidLinia IS NULL OR PidLinia ='') THEN
		RAISE noidLinia;
	END IF;

	-- Comprovaci existencia de Linia
	BEGIN 
		SELECT estat INTO estatLinia FROM LiniesComunicacio
		WHERE idLinia = PidLinia;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatLinia;		
	END;
	
	BEGIN 
		DELETE FROM Connecta cn
			WHERE (cn.idLinia=PidLinia and cn.codiContracte NOT IN (SELECT cnn.codiContracte FROM Consum cnn));
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noConnecta;
	END;
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noidLinia THEN
				resultat := 'ERROR: <idLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLinia  THEN
				resultat := 'ERROR: No s ha trobat la linia de comunicacio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoConnecta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o hi ha hagut consums a travs d aquesta connexi ';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Connecta_delLinia;
/

------------------------------------------------------
-- Conneta - delete connexions d'una centraleta
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_Connecta_delCentraleta (
	PcodiCentraleta IN Connecta.codiCentraleta%TYPE,
	PidPais IN Connecta.idPais%TYPE,
	PidLocalitat IN Connecta.idLocalitat%TYPE,
	PidAdreca IN Connecta.idAdreca%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Connecta_delCentraleta';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estatLinia CHAR(1);
	estatCentraleta CHAR(1);
	estatContracte CHAR(1);
  numConect INTEGER;
	noCodiCentraleta EXCEPTION;
	noidPais EXCEPTION;
	noidLocalitat EXCEPTION;
	noidAdreca EXCEPTION;

	noTrobatCentraleta EXCEPTION;
	noConnecta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '>';

	-- Comprovaci parmetres entrada
	IF (PcodiCentraleta IS NULL OR PcodiCentraleta ='') THEN
		RAISE noCodiCentraleta;
	END IF;

	IF (PidPais IS NULL OR PidPais ='') THEN
		RAISE noidPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat ='') THEN
		RAISE noidLocalitat;
	END IF;

	IF (PidAdreca IS NULL OR PidAdreca ='') THEN
		RAISE noidAdreca;
	END IF;

	-- Comprovaci existencia de Contracte, Centraleta i Linia
	BEGIN 
		SELECT estat INTO estatCentraleta FROM CentraletaDistribucio
		WHERE (codiCentraleta=PcodiCentraleta AND idPais=PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE NoTrobatCentraleta;		
	END;

		SELECT count(cn.codiCentraleta) INTO numConect FROM Connecta cn
			WHERE (cn.codiCentraleta=PcodiCentraleta AND cn.idPais=PidPais AND cn.idLocalitat=PidLocalitat 
			AND cn.idAdreca=PidAdreca AND cn.codiContracte NOT IN (SELECT cnn.codiContracte FROM consum cnn));

	IF numConect <=0 THEN
			RAISE noConnecta;
	END IF;
  
	
		DELETE FROM Connecta cn
			WHERE (cn.codiCentraleta=PcodiCentraleta AND cn.idPais=PidPais AND cn.idLocalitat=PidLocalitat 
			AND cn.idAdreca=PidAdreca AND cn.codiContracte NOT IN (SELECT cnn.codiContracte FROM consum cnn));
	        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <CodiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLocalitat THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta  THEN
				resultat := 'ERROR: No s ha trobat la centraleta de distribucio a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN NoConnecta  THEN
				resultat := 'ERROR: No s ha trobat la connexio o hi ha hagut consums a travs d aquesta connexi ';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Connecta_delCentraleta;
/


