------------------------------------------------------
-- LLISTAT 1 - Donada una ciutat i una data, llistat dels comptadors
--             amb consum mensual supera el 80% del consum mitj de la ciutat
--             en el mes donat
-- Parmetres d'entrada: idPais, idLocalitat, dataConsulta
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_comptadors80_llistat (
	PidPais IN Pais.idPais%TYPE,
	PidLocalitat IN Localitat.idLocalitat%TYPE,
	dataConsulta IN Consum.dataLectura%TYPE,
	resultat OUT NOCOPY Varchar2
)
IS
	procediment VARCHAR2(80) := 'p_comptadors80_llistat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
    
	v_StartDate date; 
	v_EndDate date; 
	dataLec date;
	codiCon INTEGER;
	potenciaCon NUMBER(10,2);
	cons NUMBER(10,2);
	mitjana NUMBER(10,2);
	percentat NUMBER(5,2);
    
	nomPais VARCHAR2(50);
	nomLocalitat VARCHAR2(120);
    
	err_num NUMBER;
	err_msg VARCHAR2(255);

	noPais EXCEPTION;
	noLoc	EXCEPTION;
	noData EXCEPTION;
  

-- Cursor on s'emmagatzema el resultat de la consulta
	CURSOR c_consum IS
	SELECT cs.codiContracte,cp.potenciaContractada,(100*sum(cs.consum)/(SELECT sum(cs2.consum)/count(distinct cs2.codiContracte)
                                                                            FROM consum cs2, comptador cp2 
                                                                            WHERE (cs2.dataLectura >=  v_StartDate 
                                                                                  AND cs2.dataLectura <=  v_EndDate AND cs2.codiContracte=cp2.codiContracte 
                                                                                  AND cp2.idPais=PidPais AND cp2.idLocalitat=PidLocalitat))) as percentatge
	FROM consum cs, comptador cp 
	WHERE (cs.dataLectura >=  v_StartDate AND cs.dataLectura <=  v_EndDate 
		AND cs.codiContracte=cp.codiContracte AND cp.idPais=PidPais AND cp.idLocalitat=PidLocalitat)
	GROUP BY cs.codiContracte,cp.potenciaContractada
	HAVING (100*sum(cs.consum)/(SELECT sum(cs2.consum)/count(distinct cs2.codiContracte)
                                    FROM consum cs2, comptador cp2 
                                  WHERE (cs2.dataLectura >=  v_StartDate AND cs2.dataLectura <=  v_EndDate 
                                  AND cs2.codiContracte=cp2.codiContracte AND cp2.idPais=PidPais AND cp2.idLocalitat=PidLocalitat)))>80.0;
BEGIN
	resultat:='OK';
	-- Verifiquem parmetres d'entrada

	IF (PidPais IS NULL OR PidPais ='') THEN
		RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat=0) THEN
		RAISE noLoc;
  END IF;
	IF (dataConsulta IS NULL OR dataConsulta='') THEN
		RAISE noData;
	END IF;



	-- A partir de la data de lectura determinem el perode de la consulta (del dia 1 del mes al dia 30)
	SELECT TO_DATE ('01-' || TO_CHAR (ADD_MONTHS (dataConsulta, 0),'mon-yyyy')) into v_StartDate from dual;
	SELECT TO_DATE ('01-' || TO_CHAR (ADD_MONTHS (dataConsulta, 1),'mon-yyyy')) into v_EndDate from dual;

	-- Recuperem el nom del pais
	SELECT nomPais into nomPais from pais where idPais=PidPais;

	-- Recuperem el nom de la ciutat
	SELECT nomLocalitat into nomLocalitat from localitat where idPais=PidPais and idLocalitat=PidLocalitat;    

	-- Capalera del llistat
	DBMS_OUTPUT.PUT_LINE('');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('| Llistat de comptadors que superen el 80% del consum |');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('Pais: '||nomPais);		    
	DBMS_OUTPUT.PUT_LINE('Localitat: '||nomLocalitat);		    
	DBMS_OUTPUT.PUT_LINE('Data inici perode: '||v_StartDate);		    
	DBMS_OUTPUT.PUT_LINE('Data fi perode: '||v_EndDate);		    
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
    
	DBMS_OUTPUT.PUT_LINE(RPAD('Codi Contracte',15)||RPAD(' ',5)||RPAD('Potencia Contr.',15)||RPAD(' ',5)||RPAD('% Consum',15));
	DBMS_OUTPUT.PUT_LINE(RPAD('--------------',15)||RPAD(' ',5)||RPAD('---------------',15)||RPAD(' ',5)||RPAD('--------',15));
 	BEGIN
	-- Tractament del cursor
		OPEN c_consum;
		LOOP
		FETCH c_consum INTO codiCon, potenciaCon, percentat;	
		EXIT WHEN c_consum%NOTFOUND;
			DBMS_OUTPUT.PUT_LINE(RPAD(codiCon,15)||RPAD(' ',5)||RPAD(potenciaCon,15)||RPAD(' ',5)||RPAD(percentat,15));
		END LOOP;
		CLOSE c_consum;
	DBMS_OUTPUT.PUT_LINE(' ');		  
	END;
	
	-- Execuci correcte
	p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION 
	WHEN noPais THEN
		resultat := 'ERROR: <idPais> inexistent';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN noLoc THEN
		resultat := 'ERROR: <idLoc> inexistent';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN noData THEN
		resultat := 'ERROR: <dataLectura> inexistent';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);
END p_comptadors80_llistat;
/


------------------------------------------------------
-- LLISTAT 2 - Centraletes que distribueixen ms energia
-- Parmetres d'entrada: -
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_MesEnergia_llistat (
	resultat OUT NOCOPY Varchar2
)
IS
	procediment VARCHAR2(80) := 'p_MesEnergia_llistat ';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
 
	err_num NUMBER;
	err_msg VARCHAR2(255);
    
	codiCentr INTEGER;
	iPais VARCHAR2(100);
	iLoc VARCHAR2(100);
	iAd INTEGER;
	capac NUMBER(10,2);
	suma NUMBER(10,2);
	iLoc2 INTEGER;
	iPais2 CHAR(2);
	nom VARCHAR2(100);
	numc INTEGER;
  

	-- cursor amb el resultat de la consulta
	CURSOR c_centraleta IS
      	SELECT cc.codiCentraleta, p.nomPais, l.nomLocalitat, cc.idAdreca, ct.capacitatCentraleta, sum(con.consum) 
      	FROM centraletaDistribucio ct, connecta cc, pais p, localitat l, consum con
       	WHERE ct.idPais = cc.idPais AND cc.idLocalitat=ct.idLocalitat AND cc.idAdreca=ct.idAdreca AND cc.codiCentraleta=ct.codiCentraleta
            	and p.idPais = cc.idPais and l.idPais = cc.idPais and l.idLocalitat=cc.idLocalitat and cc.codiContracte = con.codiContracte
        	GROUP BY cc.codiCentraleta,p.nomPais, l.nomLocalitat, cc.idAdreca,ct.capacitatCentraleta
        	ORDER BY sum(con.consum) DESC;
       
BEGIN
	resultat:='OK';
    
	-- Capalera de la consulta
	DBMS_OUTPUT.PUT_LINE('');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('| Llistat de centraletes que distribueixen ms  |');
	DBMS_OUTPUT.PUT_LINE('| energia                                       |');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');


	BEGIN
	-- Tractament del cursor
		OPEN c_centraleta;
      
      	DBMS_OUTPUT.PUT_LINE(RPAD('Pais',15)||RPAD(' ',5)||RPAD('Localitat',15)||RPAD(' ',5)||RPAD('Adreca',15)||RPAD(' ',5)||
              RPAD('Energia',15)||RPAD(' ',5)||RPAD('Capacitat',150)||RPAD(' ',5));
      	DBMS_OUTPUT.PUT_LINE(RPAD('----',15)||RPAD(' ',5)||RPAD('---------',15)||RPAD(' ',5)||RPAD('------',15)||RPAD(' ',5)||
              RPAD('-------',15)||RPAD(' ',5)||RPAD('---------',15)||RPAD(' ',5));              

	-- Loop sobre el cursor
		LOOP
		FETCH c_centraleta INTO codiCentr, iPais, iLoc, iAd, capac, suma;	
	-- Parem quan no hi hagi dades o haguem arribat a 10 dades
		EXIT WHEN c_centraleta%NOTFOUND OR c_centraleta%ROWCOUNT >10;
			DBMS_OUTPUT.PUT_LINE(RPAD(iPais,15)||RPAD(' ',5)||RPAD(iLoc,15)||RPAD(' ',5)||RPAD(iAd,15)||RPAD(' ',5)||
						RPAD(suma,15)||RPAD(' ',5)||RPAD(capac,15)||RPAD(' ',5));
		END LOOP;

		CLOSE c_centraleta;
	END;
  -- Execuci correcte
	p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION 
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);    
END p_MesEnergia_llistat;
/

------------------------------------------------------
-- LLISTAT 3 - Linies ms carregades
-- Parmetres d'entrada: -
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_MesCarregada_llistat (
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_MesCarregada_llistat';
  entrada VARCHAR2(4000) := ' ';
  sortida VARCHAR2(4000) := ' ';
 
  err_num NUMBER;
  err_msg VARCHAR2(255);
  
  linia INTEGER;
  carrega NUMBER(10,2);
  mesos INTEGER;
  capacitat NUMBER(10,2);
  incr NUMBER(10,2);
  percentatge NUMBER(10,2);
  n INTEGER;
  
-- Linies que connecten centrals de producci amb centraletes de distribuci
    CURSOR c_linia IS 
select distinct als.idLinia, (select  sum(cs.consum) 
                      		from consum cs
                      		where cs.codiContracte IN (select distinct cn.codiContracte
                                                	from connecta cn
                                                	inner join alimenta al
                                                      	on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                                	and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                                	where al.idLinia=als.idLinia)) as carrega,  (select  count(cs.consum) 
					from consum cs
					where cs.codiContracte IN (select distinct cn.codiContracte
									from connecta cn
									inner join alimenta al
										on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                                	and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                                	where al.idLinia=als.idLinia)) as mesos,
			ls.capacitatLinia, cp.energiaMax-ls.capacitatLinia as incrementEnergia
from alimenta als, centralProduccio cp, liniesComunicacio ls
where als.codiCentral=cp.codiCentral and als.idLinia=ls.idLinia
order by carrega desc;


-- Linies que connecten centraletes de distribuci amb comptadors
CURSOR c_linia2 IS      
    select distinct cn.idLinia, 
    (select sum(cs.consum) from consum cs, connecta cns where cns.codiContracte=cs.codiContracte and cns.idLinia=cn.idLinia) as carrega,
    (select count(cs.consum) from consum cs, connecta cns where cns.codiContracte=cs.codiContracte and cns.idLinia=cn.idLinia) as mesos,
    lc.capacitatLinia,SUM(distinct cp.energiaMax)-lc.capacitatLinia as incrementE

    from connecta cn
      inner join alimenta al
        on cn.idPais=al.idPais and cn.codiCentraleta=al.codiCentraleta and cn.idLocalitat = al.idLocalitat and cn.idAdreca = al.idAdreca
    inner join centralProduccio cp
      on al.codiCentral=cp.codiCentral
    inner join liniescomunicacio lc
      on cn.idLinia=lc.idLinia
    group by cn.idLinia,lc.capacitatLinia
    order by carrega desc;      
   
BEGIN
  
	resultat:='OK';
	
	-- Capalera del llistat
	DBMS_OUTPUT.PUT_LINE('');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('| Linies ms carregades                         |');
	DBMS_OUTPUT.PUT_LINE('|                                               |');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');

	DBMS_OUTPUT.PUT_LINE(RPAD('IdLinia',10)||RPAD(' ',5)||RPAD('% Carrega',10)||RPAD(' ',5)||RPAD('Possible Increment Energia',30));
	DBMS_OUTPUT.PUT_LINE(RPAD('-------',10)||RPAD(' ',5)||RPAD('---------',10)||RPAD(' ',5)||RPAD('--------------------------',30));    

	BEGIN
	-- Tractament del cursor
		n:=0;
		OPEN c_linia;
		
		LOOP
		FETCH c_linia INTO linia, carrega,mesos,capacitat, incr;	
		-- parem quan no hi hagi dades o haguem arribat a 10
		EXIT WHEN c_linia%NOTFOUND OR c_linia%ROWCOUNT >10;
			-- clcul del percentatge
			percentatge:=100*carrega/(capacitat*mesos);
			DBMS_OUTPUT.PUT_LINE(RPAD(linia,10)||RPAD(' ',5)||RPAD(percentatge,10)||RPAD(' ',5)||RPAD(incr,30));          
			n:=n+1;
		END LOOP;
			
		CLOSE c_linia;
		
	-- Tractament del cursor2
		OPEN c_linia2;
      
		LOOP
		FETCH c_linia2 INTO linia, carrega,mesos,capacitat, incr;	
		EXIT WHEN c_linia2%NOTFOUND OR n >10;
			percentatge:=100*carrega/(capacitat*mesos);
			DBMS_OUTPUT.PUT_LINE(RPAD(linia,10)||RPAD(' ',5)||RPAD(percentatge,10)||RPAD(' ',5)||RPAD(incr,10));          
			n:=n+1;
		END LOOP;

		CLOSE c_linia2;   
	END;

	-- Execuci correcte
	p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION 
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);    

END p_MesCarregada_llistat;
/

------------------------------------------------------
-- LLISTAT 4 - Clients amb servei en alta
-- Parmetres d'entrada: -
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_ServeiAlta_llistat (
	resultat OUT NOCOPY Varchar2
	)
IS
	procediment VARCHAR2(80) := 'p_ServeiAlta_llistat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
 
	err_num NUMBER;
	err_msg VARCHAR2(255);
  
	dni VARCHAR(10);
	codiContr INTEGER;
	modelC VARCHAR2(255);

-- Cursor amb el resultat de la consulta:
-- Noms es proporcionen clients actius que estiguin connectats en alta a elements actius

	CURSOR c_clientsAlta IS 
  		select ct.dni,ct.codiContracte,ct.modelComptador
		from comptador ct
		inner join client cl
			on cl.dni=ct.dni and cl.estat='A'
		where ct.estat='A' and ct.codiContracte IN (
            -- Comptadors que tenen servei d'alta
			select cn.codiContracte
			from  connecta cn
			inner join  alimenta al
				on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
            		and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca
			inner join centralProduccio cp
				on cp.codiCentral = al.codiCentral and cp.estat='A'
			inner join centraletaDistribucio cd
				on  al.codiCentraleta = cd.codiCentraleta and al.idPais = cd.idPais 
			          and al.idLocalitat = cd.idLocalitat and al.idAdreca = cd.idAdreca and cd.estat='A'
			inner join LiniesComunicacio lc
				on al.idLinia=lc.idLinia and lc.estat='A'
			group by cn.codiContracte
			having count(distinct cn.idLinia)>1 and count(distinct al.codiCentral)>1 and
                      count(distinct to_char(al.codiCentraleta||al.idPais||al.idLocalitat||al.idAdreca))>1
                      and count(distinct al.idLinia)>1)
		order by ct.codiContracte asc;
       
BEGIN
	
    resultat:='OK';
    
-- Capalera llistat
	DBMS_OUTPUT.PUT_LINE('');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('| Clients amb servei en alta                    |');
	DBMS_OUTPUT.PUT_LINE('|                                               |');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');

	DBMS_OUTPUT.PUT_LINE(RPAD('DNI client',10)||RPAD(' ',5)||RPAD('Codi Contracte',15)||RPAD(' ',5)||RPAD('Model Comptador',15));
	DBMS_OUTPUT.PUT_LINE(RPAD('----------',10)||RPAD(' ',5)||RPAD('--------------',15)||RPAD(' ',5)||RPAD('---------------',15));    
 	BEGIN
	-- Tractament del cursor
		OPEN c_clientsAlta;
    
		LOOP
		FETCH c_clientsAlta INTO dni, codiContr, modelC;	
		EXIT WHEN c_clientsAlta%NOTFOUND; 
			DBMS_OUTPUT.PUT_LINE(RPAD(dni,10)||RPAD(' ',5)||RPAD(codiContr,15)||RPAD(' ',5)||RPAD(modelC,15));
		END LOOP;
		CLOSE c_clientsAlta;
	END;
  -- Execuci correcte
	p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION 
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);        
END p_ServeiAlta_llistat;
/

------------------------------------------------------
-- LLISTAT 5 - Consum dels comptadors d'una central i Energia produda
-- Parmetres d'entrada: codiCentral, dataInicial, dataFinal
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Central_llistat (
  PcodiCentral IN centralProduccio.codiCentral%TYPE,
  PdataInicial IN produccio.dataLectura%TYPE,
  PdataFinal IN produccio.dataLectura%TYPE,
  resultat OUT NOCOPY Varchar2
	)
IS

	procediment VARCHAR2(80) := 'p_Central_llistat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
 
	err_num NUMBER;
	err_msg VARCHAR2(255);
  
	consumComp NUMBER(10,2);
	producc NUMBER(10,2);
	
      noCentral EXCEPTION;
      noDataInicial EXCEPTION;
      noDataFinal EXCEPTION;
      errorDates EXCEPTION;

BEGIN

	resultat:='OK';
	
	-- verifiquem dades d'entrada
	IF (PcodiCentral IS NULL OR PcodiCentral=0) THEN
		RAISE noCentral;
	END IF;
	
	IF (PdataInicial IS NULL OR PdataInicial='') THEN
		RAISE noDataInicial;
	END IF;
	
	IF (PdataFinal IS NULL OR PdataFinal='') 	THEN
		RAISE noDataFinal;
	END IF;
	
	IF (PdataFinal < PdataInicial) THEN
		RAISE errorDates;
	END IF;
	
	DBMS_OUTPUT.PUT_LINE('');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('| Central: consum dels comptadors +             |');
	DBMS_OUTPUT.PUT_LINE('|          + producci de la central            |');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');

	BEGIN
  
		-- CONSUM DELS COMPTADORS DE LA CENTRAL EN EL PERIODE DE TEMPS
      	select  sum(cs.consum) into consumComp
      	from consum cs
      	where cs.codiContracte IN (
			select distinct cn.codiContracte
			from connecta cn
      	     	inner join alimenta al
				on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
				and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
			where al.codiCentral=PcodiCentral)
			and cs.dataLectura <=PdataFinal and cs.dataLectura >= PdataInicial;
    
    		-- PRODUCCIO DE LA CENTRAL EN EL PERIODE DE TEMPS
      	select sum(p.produccioMes) into producc
      	from produccio p
      	where p.codiCentral = PcodiCentral and p.dataLectura <=PdataFinal and p.dataLectura >= PdataInicial;

		DBMS_OUTPUT.PUT_LINE('Central: '||PcodiCentral);
		DBMS_OUTPUT.PUT_LINE('Consum comptadors: '||consumComp);	          
		DBMS_OUTPUT.PUT_LINE('Energia produda: '||producc);	          
		DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
	END;
	-- Execuci correcte
	  p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION 
	WHEN noCentral THEN
		resultat := 'ERROR: <codiCentral> inexistent ' ;
		p_log_insert(procediment, entrada, sortida, resultat);     
	WHEN noDataInicial THEN
		resultat := 'ERROR: <dataInicial> inexistent ' ;
		p_log_insert(procediment, entrada, sortida, resultat);     
	WHEN noDataFinal THEN
		resultat := 'ERROR: <dataFinal> inexistent ' ;
		p_log_insert(procediment, entrada, sortida, resultat);     
	WHEN errorDates THEN
		resultat := 'ERROR: data inicial posterior a data final' ;
		p_log_insert(procediment, entrada, sortida, resultat);     
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);     
END p_Central_llistat;
/

------------------------------------------------------
-- LLISTAT 6 - Percentatge de lectures presencials i telemtiques
-- Parmetres d'entrada: 
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Lectures_llistat (
  resultat OUT NOCOPY Varchar2
	)
IS

  procediment VARCHAR2(80) := 'p_Lectures_llistat';
  entrada VARCHAR2(4000) := ' ';
  sortida VARCHAR2(4000) := ' ';
 
  err_num NUMBER;
  err_msg VARCHAR2(255);
  presen INTEGER;
  totals INTEGER;    
  percentatge NUMBER(5,2);
BEGIN
    
    resultat:='OK';
	
	DBMS_OUTPUT.PUT_LINE('');
	DBMS_OUTPUT.PUT_LINE('----------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('| Percentatge lectures telemtiques i presencials  |');
	DBMS_OUTPUT.PUT_LINE('----------------------------------------------------');

	BEGIN

-- Nombre de lectures de comptadors fetes
		select count(cs.tipusLectura) into totals from consum cs;
-- Lectures presencials
		select count(cs.tipusLectura) into presen from consum cs where cs.tipusLectura='P';

        
		percentatge:=100*(totals-presen)/totals;
          
		DBMS_OUTPUT.PUT_LINE('Lectures telemtiques: '||percentatge||'%');
		percentatge:=100*presen/totals;
		DBMS_OUTPUT.PUT_LINE('Lectures presencials:  '||percentatge||'%');         
		DBMS_OUTPUT.PUT_LINE('Lectures totals: '||totals);	          
    DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
	END;
  -- Execuci correcte
	p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION 
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);         
END p_Lectures_llistat;
/

------------------------------------------------------
-- LLISTAT 7 - Llistat de comptadors amb els anys d'antiguitat demanats
-- Parmetres d'entrada: anys_antiguitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Antiguitat_llistat (
  Panys IN NUMBER,
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_Antiguitat_llistat';
  entrada VARCHAR2(4000) := ' ';
  sortida VARCHAR2(4000) := ' ';
 
  err_num NUMBER;
  err_msg VARCHAR2(255);
  
  codi INTEGER;
  CURSOR c_comptadors IS 
      select cp.CodiContracte
      from comptador cp
      where cp.dataFabricacio <= ADD_MONTHS(SYSDATE,-12*Panys) and cp.estat='A';

BEGIN
    
    resultat:='OK';
    
	DBMS_OUTPUT.PUT_LINE('');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------------');
	DBMS_OUTPUT.PUT_LINE('| Comptadors amb l antiguitat superior a '||Panys||' anys     |');
	DBMS_OUTPUT.PUT_LINE('|                                                     |');
	DBMS_OUTPUT.PUT_LINE('-------------------------------------------------------');
	-- Tractament del cursor
			OPEN c_comptadors;
    
			LOOP
				FETCH c_comptadors INTO codi;	
				EXIT WHEN c_comptadors%NOTFOUND; 
          DBMS_OUTPUT.PUT_LINE('Codi Comptador: '||codi);
      END LOOP;
			CLOSE c_comptadors;
    DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
    
  -- Execuci correcte  
  p_log_insert(procediment, entrada, sortida, resultat);
  EXCEPTION 
  			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);    
END p_Antiguitat_llistat;
/




