------------------------------------------------------
-- ESTADISTICA 1 - Donada una central de producci
--                 consum dels comptadors que depenen de la central
-- Parmetres d'entrada: codiCentral
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E1_read (
	PcodiCentral IN E1.codiCentral%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
	procediment VARCHAR2(80) := 'p_E1_read';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
	cons NUMBER(10,2);
  
	noCodiCentral EXCEPTION;
	noCentral EXCEPTION;
  
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
    
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '>' ;
      
	IF (PcodiCentral IS NULL OR PcodiCentral  = '') THEN
		RAISE noCodiCentral ;
	END IF;
    
	BEGIN 
		SELECT consum into cons from E1 where codiCentral=PcodiCentral;
		DBMS_OUTPUT.PUT_LINE('');
		DBMS_OUTPUT.PUT_LINE('LLISTAT ESTADSTIC 1. CONSUM DELS COMPTADORS DE LA CENTRAL: '||RPAD(PcodiCentral,10));
		DBMS_OUTPUT.PUT_LINE('----------------------------------------------------------------------');
	  DBMS_OUTPUT.PUT_LINE('Consum = '||cons);	
	  DBMS_OUTPUT.PUT_LINE('');		    
  EXCEPTION
		WHEN NO_DATA_FOUND THEN
		RAISE NoCentral;
	END;
    
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
	WHEN noCodiCentral THEN
		resultat := 'ERROR: <CodiCentral> inexistent';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN NoCentral THEN
		resultat := 'ERROR: No hi ha dades de la central';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);        
END p_E1_read;
/

------------------------------------------------------
-- ESTADISTICA 1 - Donada una central de producci
--                 consum dels comptadors que depenen de la central
-- Parmetres d'entrada: codiCentral, consum
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E1_insert (
  PcodiContracte IN consum.codiContracte%TYPE,
  Pconsum IN consum.consum%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
	procediment VARCHAR2(80) := 'p_E1_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
	codi INTEGER;
	cons NUMBER(10,2);
  
	err_num NUMBER;
	err_msg VARCHAR2(255);
  
-- Centrals del comptador
  CURSOR c_central IS        
    select distinct al.codiCentral 
    from alimenta al
    inner join connecta cn
        on al.codiCentraleta=cn.codiCentraleta and al.idPais=cn.idPais and al.idLocalitat=cn.idlocalitat and al.idAdreca=cn.idAdreca
    where cn.codiContracte=PcodiContracte;
  
BEGIN
    
	resultat := 'OK';
	entrada := '<' || PcodiContracte || '> <' || Pconsum || '>' ;
  BEGIN
	-- tractament del cursor
    OPEN c_central;
  
    LOOP
    FETCH c_central INTO codi;	
    EXIT WHEN c_central%NOTFOUND; 
      BEGIN
        SELECT consum INTO cons from E1 where codiCentral = codi;
        cons:=cons+Pconsum;
        UPDATE E1 SET
          consum=cons
        WHERE codiCentral=codi;
      EXCEPTION
      WHEN NO_DATA_FOUND THEN
        INSERT INTO E1(codiCentral,consum) VALUES (codi,Pconsum);
      END;
	END LOOP;
		CLOSE c_central;
	END;

	-- Execuci correcte del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

EXCEPTION     
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);   
END p_E1_insert;
/

------------------------------------------------------
-- ESTADISTICA 2 - Donada una linia i un any concret
--                 consum dels comptadors que s'alimenten d'aquesta linia
-- Parmetres d'entrada: idLinia, any
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E2_read (
	PidLinia IN E2.idLinia%TYPE,
	Pany IN E2.ao%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
	procediment VARCHAR2(80) := 'p_E2_read';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
	en NUMBER(10,2);
  
	noidLinia EXCEPTION;
	noAny EXCEPTION;
	noDada EXCEPTION;  
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
    
	resultat := 'OK';
	entrada := '<' || PidLinia || '> <' || Pany || '>' ;
      
	IF (PidLinia IS NULL OR PidLinia = 0) THEN
		RAISE noidLinia ;
	END IF;

	IF (Pany IS NULL OR Pany= 0) THEN
		RAISE noAny;
	END IF;
    
	BEGIN 

    SELECT energiaConsumida INTO en 
    FROM E2 
    WHERE idLinia=PidLinia AND ao=Pany;
    DBMS_OUTPUT.PUT_LINE('');
		DBMS_OUTPUT.PUT_LINE('LLISTAT ESTADSTIC 2. CONSUM DELS COMPTADORS DE LA LINIA: '||RPAD(PidLinia,10)||'DURANT L ANY: '||RPAD(Pany,4));
		DBMS_OUTPUT.PUT_LINE('----------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE('Consum = '||en);	
    DBMS_OUTPUT.PUT_LINE('');    
	EXCEPTION
      	WHEN NO_DATA_FOUND THEN
			RAISE NoDada;
	END;
    
	-- Execuci correcte del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
	WHEN noidLinia THEN
		resultat := 'ERROR: <idLinia> inexistent';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN noAny THEN
		resultat := 'ERROR: <Anyl> inexistent';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN NoDada THEN
		resultat := 'ERROR: No hi ha dades de la linia per aquest any';
		p_log_insert(procediment, entrada, sortida, resultat);
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);        
END p_E2_read;
/

------------------------------------------------------
-- ESTADISTICA 2 - Update de l'estadistica 2
-- Parmetres d'entrada: codiContracte, dataLectura
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E2_insert (
	PdataLect IN consum.dataLectura%TYPE,
	PcodiContracte IN consum.codiContracte%TYPE,
	Pconsum IN consum.consum%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
	procediment VARCHAR2(80) := 'p_E2_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
	lin INTEGER;
	consActual NUMBER(10,2);
	anyLectura INTEGER;
  
	err_num NUMBER;
	err_msg VARCHAR2(255);

-- Cursors amb les linies associades al contracte que ha tingut el consum
-- Linies Central - Centraleta
	CURSOR c_linies IS
		select distinct al.idLinia 
		from alimenta al
		inner join connecta cn
			on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
			and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
		where cn.codiContracte =PcodiContracte;
    
-- Linies Centraleta - Comptador
	CURSOR c_linies2 IS
		select distinct cn.idLinia 
		from connecta cn
		where cn.codiContracte =PcodiContracte;
BEGIN

	resultat := 'OK';
	entrada := '<' || PdataLect || '> <' ||PcodiContracte || '> <' || Pconsum ||'>' ;

-- Obtenim l'any en el que s'ha fet la lectura    
	SELECT TO_NUMBER(TO_CHAR(PdataLect,'YYYY')) INTO anyLectura FROM dual;  
    
	BEGIN
    
    -- tractament del cursor linies central - centraleta
		OPEN c_linies;
		LOOP
		FETCH c_linies INTO lin;	
		EXIT WHEN c_linies%NOTFOUND; 
			BEGIN
				-- Recuperem el consum actual de la linia
				SELECT energiaConsumida INTO consActual FROM E2 WHERE idLinia=lin AND ao=anyLectura;
        -- Afegim el nou consum
	  			consActual:=consActual+Pconsum;
				-- Actualitzem l'estadstica
          UPDATE E2 SET
            energiaConsumida=consActual
          WHERE idLinia=lin and ao=anyLectura;
      EXCEPTION
        -- Si s el primer consum de la linia, l'afegim a la taula
        WHEN NO_DATA_FOUND THEN
          INSERT INTO e2(ao,idLinia,energiaConsumida) VALUES (anyLectura,lin,Pconsum);
      END;
        
		END LOOP;
    
		CLOSE c_linies;

    -- tractament del cursor linies centraleta - comptador
		OPEN c_linies2;
		LOOP
		FETCH c_linies2 INTO lin;	
		EXIT WHEN c_linies2%NOTFOUND; 
			BEGIN
				-- Recuperem el consum actual de la linia
				SELECT energiaConsumida INTO consActual FROM E2 WHERE idLinia=lin AND ao=anyLectura;
          -- Afegim el nou consum
          consActual:=consActual+Pconsum;
          -- Actualitzem l'estadstica
          UPDATE E2 SET
  					energiaConsumida=consActual
    			WHERE idLinia=lin and ao=anyLectura;
      EXCEPTION
        -- Si s el primer consum de la linia, l'afegim a la taula
        WHEN NO_DATA_FOUND THEN
          INSERT INTO e2(ao,idLinia,energiaConsumida) VALUES (anyLectura,lin,Pconsum);
		END;
        
		END LOOP;
		CLOSE c_linies2;

    END;
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);

EXCEPTION     
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);   
END p_E2_insert;
/

------------------------------------------------------
-- ESTADISTICA 3 - Lnia ms carregada
-- Parmetres d'entrada: --
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E3_read (
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E3_read';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

  lin INTEGER;
  noDada EXCEPTION;  
  err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
    
  resultat := 'OK';
  entrada := '< >' ;
    
  BEGIN 
    SELECT idLinia into lin from E3;
    DBMS_OUTPUT.PUT_LINE('LLISTAT ESTADSTIC 3. LINIA MS CARREGADA.');
    DBMS_OUTPUT.PUT_LINE('------------------------------------------');    
    DBMS_OUTPUT.PUT_LINE('La linia ms carregada s la: '||lin);		    
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RAISE NoDada;
    END;
    
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN NoDada THEN
				resultat := 'ERROR: No hi ha dades en l estadistica 3';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);        
END p_E3_read;
/
------------------------------------------------------
-- ESTADISTICA 3 - Update de l'estadistica 3
-- Parmetres d'entrada: -
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E3_insert (
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E3_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
  lin INTEGER;
  carr NUMBER(10,2);
  
  err_num NUMBER;
	err_msg VARCHAR2(255);

-- Linies Central - Centraleta
-- No considero les linies Centraleta - Comptador pq segur que transporten menys energia que les linies Central Comptador
  CURSOR c_linies IS
      select distinct als.idLinia, (select  sum(cs.consum) 
                                    from consum cs
                                    where cs.codiContracte IN (select distinct cn.codiContracte
                                                                from connecta cn
                                                                inner join alimenta al
                                                                  on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                                                  and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                                                where al.idLinia=als.idLinia)) as carrega
      from alimenta als
      group by als.idLinia
      order by carrega desc;
      
BEGIN

    resultat := 'OK';
    entrada := '< >' ;
   
    BEGIN
   
      -- tractament del cursor
      OPEN c_linies;
      LOOP
				FETCH c_linies INTO lin,carr;	
				EXIT WHEN c_linies%NOTFOUND OR c_linies%ROWCOUNT > 1; 
        
        BEGIN
          delete from e3;
          insert into e3(idLinia) VALUES (lin);
        END;
        
			END LOOP;
			CLOSE c_linies;
    END;
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);   
END p_E3_insert;
/
------------------------------------------------------
-- ESTADISTICA 4 - Percentatge de linies amb consum >50%
-- Parmetres d'entrada: any
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E4_read (
  Pao IN E4.ao%TYPE,
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E4_read';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

  percenta NUMBER(5,2);
  noDada EXCEPTION;  
  err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
    
    resultat := 'OK';
    entrada := '<'||Pao||'>' ;
    
    BEGIN 
      SELECT percentatge into percenta from E4 where ao=Pao;
      DBMS_OUTPUT.PUT_LINE('');
      DBMS_OUTPUT.PUT_LINE('LLISTAT ESTADSTIC 4. PERCENTATGE DE LINIES AMB CONSUM > 50% DEL CONSUM TOTAL DE L ANY.');
      DBMS_OUTPUT.PUT_LINE('---------------------------------------------------------------------------------------');    
      DBMS_OUTPUT.PUT_LINE('Any '||Pao||' tenen un consum superior al 50% del consum total un '||percenta||'% del total de linies');		    
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RAISE NoDada;
    END;
    
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN NoDada THEN
				resultat := 'ERROR: No hi ha dada en l estadistica 4';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);        
END p_E4_read;
/

------------------------------------------------------
-- ESTADISTICA 4 - Update de l'estadistica 4
-- Parmetres d'entrada: dataLectura
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E4_insert (
  pDataLectura IN consum.dataLectura%TYPE,
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E4_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
  anys INTEGER;
  lin INTEGER;
  nlin INTEGER;
  ctotal NUMBER(10,2);
  clinia NUMBER(10,2);
  carr NUMBER(10,2);
  percenta NUMBER(10,2);
  cTotalLin INTEGER;
  err_num NUMBER;
	err_msg VARCHAR2(255);
  
-- Calculo el consum de totes les linies l'any donat
-- No considero les linies Centraleta - Comptador
  CURSOR c_linies IS
      select distinct als.idLinia, (select  sum(cs.consum) 
                                    from consum cs
                                    where (select to_number(TO_CHAR(cs.dataLectura,'yyyy')) from dual)=anys 
                                          and cs.codiContracte IN (select distinct cn.codiContracte
                                                                from connecta cn
                                                                inner join alimenta al
                                                                  on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
                                                                  and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
                                                                where al.idLinia=als.idLinia)) as carrega
      from alimenta als
      group by als.idLinia
      order by carrega desc;
-- Total linies que connecten Centrals - Centraletes      
  CURSOR c_totalLinies IS
    select count(distinct al.idLinia) from alimenta al;
-- Consum total dels comptadors
  CURSOR c_consumTotal IS
      select  sum(cs.consum) 
      from consum cs
      where (select to_number(TO_CHAR(cs.dataLectura,'yyyy')) from dual)=anys;
    
BEGIN

    resultat := 'OK';
    entrada := '<'||PdataLectura||'>' ;
   
    SELECT to_number(TO_CHAR(PdataLectura,'yyyy')) INTO anys from dual; 
    
    BEGIN
      -- tractament dels cursors
      OPEN c_consumTotal;
      LOOP
        FETCH c_consumTotal INTO ctotal;
          EXIT WHEN c_consumTotal%NOTFOUND;
      END LOOP;
      CLOSE c_consumTotal;
    
      OPEN c_TotalLinies;
      LOOP
        FETCH c_TotalLinies INTO cTotalLin;
          EXIT WHEN c_TotalLinies%NOTFOUND;
      END LOOP;
      CLOSE c_TotalLinies;

      nlin:=0;
      
      OPEN c_linies;
      LOOP
				FETCH c_linies INTO lin,carr;	
				EXIT WHEN c_linies%NOTFOUND OR carr/ctotal < .5; 
        BEGIN
          nlin:=nlin+1;    
        END;
			END LOOP;
			CLOSE c_linies;
      
      percenta := 100*nlin/cTotalLin;
      BEGIN
        INSERT INTO E4(ao,percentatge) VALUES (anys,percenta);
        EXCEPTION
          WHEN DUP_VAL_ON_INDEX THEN
            UPDATE E4 SET
             percentatge=percenta
            WHERE ao=anys;             
          END;
        END;

    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);   
END p_E4_insert;
/


------------------------------------------------------
-- ESTADISTICA 5 - Nombre de centrals que generen menys del 30% de producci
-- Parmetres d'entrada: any
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E5_read (
  Pao IN E4.ao%TYPE,
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E5_read';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

  nCentrals INTEGER;
  noDada EXCEPTION;  
  err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
    
    resultat := 'OK';
    entrada := '<'||Pao||'>' ;
    
    BEGIN 
      SELECT centrals into nCentrals from E5 where ao=Pao;
      DBMS_OUTPUT.PUT_LINE('');
      DBMS_OUTPUT.PUT_LINE('LLISTAT ESTADSTIC 5. NOMBRE DE CENTRALS QUE GENEREN MENYS DEL 30% DE LA PRODUCCIO TOTAL DURANT L ANY.');
      DBMS_OUTPUT.PUT_LINE('------------------------------------------------------------------------------------------------------');    
      DBMS_OUTPUT.PUT_LINE('Any '||Pao||' tenen una producci inferior al 30% de la producci total, '||nCentrals||' centrals de produccio');		    
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RAISE NoDada;
    END;
    
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN NoDada THEN
				resultat := 'ERROR: No hi ha dada en l estadistica 5';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);        
END p_E5_read;
/

------------------------------------------------------
-- ESTADISTICA 5 - Update de l'estadistica 5
-- Parmetres d'entrada: dataProduccio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E5_insert (
  pDataLectura IN produccio.dataLectura%TYPE,
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E5_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
  anys INTEGER;
  cen INTEGER;
  percenta NUMBER(5,2);
  ncen INTEGER;

  err_num NUMBER;
	err_msg VARCHAR2(255);
  
  -- Clcul de la producci
  CURSOR c_centrals IS
        select p.codiCentral, 100.*sum(p.produccioMes)/(select sum(pt.produccioMes) 
                                          from produccio pt 
                                          where (select to_number(TO_CHAR(pt.dataLectura,'yyyy')) from dual)=anys) as percenta
        from produccio p
        where (select to_number(TO_CHAR(p.dataLectura,'yyyy')) from dual)=anys
        group by p.codiCentral
        having sum(p.produccioMes)/(select sum(pt.produccioMes) 
                                          from produccio pt 
                                          where (select to_number(TO_CHAR(pt.dataLectura,'yyyy')) from dual)=anys)<0.3;
      
   
BEGIN

    resultat := 'OK';
    entrada := '< >' ;
   
    SELECT to_number(TO_CHAR(PdataLectura,'yyyy')) INTO anys from dual; 
    
    BEGIN
      -- tractament dels cursors
    ncen:=0;
      
      OPEN c_centrals;
      LOOP
				FETCH c_centrals INTO cen,percenta;	
				EXIT WHEN c_centrals%NOTFOUND; 
        BEGIN
          ncen:=ncen+1;    
        END;
			END LOOP;
			CLOSE c_centrals;
      
  
      BEGIN
        INSERT INTO E5(ao,centrals) VALUES (anys,ncen);
        EXCEPTION
          WHEN DUP_VAL_ON_INDEX THEN
            UPDATE E5 SET
             centrals=ncen
            WHERE ao=anys;             
          END;
        END;

    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);   
END p_E5_insert;
/


------------------------------------------------------
-- ESTADISTICA 6 - Top ten de comptadors amb ms consum histric
-- Parmetres d'entrada: --
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E6_read (
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E6_read';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

  codi INTEGER;
  consum NUMBER(10,2);
  nCentrals INTEGER;
  noDada EXCEPTION;  
  err_num NUMBER;
	err_msg VARCHAR2(255);
  
  CURSOR c_contractes IS
    select * from E6;
	
BEGIN
    
    resultat := 'OK';
    entrada := '< >' ;
    
    BEGIN 
      DBMS_OUTPUT.PUT_LINE('');
      DBMS_OUTPUT.PUT_LINE('LLISTAT ESTADSTIC 6. TOP TEN COMPTADORS AMB MS CONSUM HISTRIC.');
      DBMS_OUTPUT.PUT_LINE('------------------------------------------');        
      DBMS_OUTPUT.PUT_LINE('  ');        
      DBMS_OUTPUT.PUT_LINE(RPAD('Codi Comptador',20)||RPAD('  ',2)||RPAD('Consum',10));		    
      DBMS_OUTPUT.PUT_LINE(RPAD('--------------',20)||RPAD('  ',2)||RPAD('------',10));		       
      
      nCentrals:=0;
      open c_contractes;
      LOOP
				FETCH c_contractes INTO codi,consum;	
				EXIT WHEN c_contractes%NOTFOUND; 
        BEGIN
            nCentrals:=nCentrals+1;
       DBMS_OUTPUT.PUT_LINE(RPAD(codi,20)||RPAD('  ',2)||RPAD(consum,10));	        
        END;
			END LOOP;
			CLOSE c_contractes;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
          RAISE noDada;
    END;
      
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN NoDada THEN
				resultat := 'ERROR: No hi ha dada en l estadistica 6';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);        
END p_E6_read;
/

------------------------------------------------------
-- ESTADISTICA 6 - Update estadistica 6
-- Parmetres d'entrada: dataProduccio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E6_insert (
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E6_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
  codi INTEGER;
  consum NUMBER(10,2);


  err_num NUMBER;
	err_msg VARCHAR2(255);
  
  CURSOR c_contractes IS
      select c.codiContracte, sum(c.consum)
      from consum c
      group by c.codiContracte
      order by sum(c.consum)desc;
      
   
BEGIN

    resultat := 'OK';
    entrada := '< >' ;
   
      -- tractament dels cursors
    delete from e6;
      
      OPEN c_contractes;
      LOOP
				FETCH c_contractes INTO codi,consum;	
				EXIT WHEN c_contractes%NOTFOUND OR c_contractes%ROWCOUNT >10; 
        BEGIN
          INSERT INTO E6(codiContracte, consumHistoric) VALUES (codi,consum);    
        END;
			END LOOP;
			CLOSE c_contractes;
      
 
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN DUP_VAL_ON_INDEX  THEN
				resultat := 'ERROR: codi contracte repetit';
				p_log_insert(procediment, entrada, sortida, resultat);  
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);   
END p_E6_insert;
/

------------------------------------------------------
-- ESTADISTICA 7 - Consum mig de tots els clients
-- Parmetres d'entrada: --
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E7_read (
  resultat OUT NOCOPY Varchar2
	)
IS
  	procediment VARCHAR2(80) := 'p_E7_read';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	codi INTEGER;
	consum NUMBER(10,2);
	nCentrals INTEGER;
	noDada EXCEPTION;  
	err_num NUMBER;
	err_msg VARCHAR2(255);
  
	
BEGIN
    
    resultat := 'OK';
    entrada := '< >' ;
    
    BEGIN 
      Select consumMig into consum from E7;
      DBMS_OUTPUT.PUT_LINE('');
      DBMS_OUTPUT.PUT_LINE('LLISTAT ESTADSTIC 7.  CONSUM MIG MENSUAL.');
      DBMS_OUTPUT.PUT_LINE('------------------------------------------');      
      DBMS_OUTPUT.PUT_LINE('Consum: '||consum);		    

    EXCEPTION
      WHEN NO_DATA_FOUND THEN
          RAISE noDada;
    END;
      
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
			WHEN NoDada THEN
				resultat := 'ERROR: No hi ha dada en l estadistica 7';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);        
END p_E7_read;
/

------------------------------------------------------
-- ESTADISTICA 7 - Update estadistica 7
-- Parmetres d'entrada: 
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_E7_insert (
  resultat OUT NOCOPY Varchar2
	)
IS
  procediment VARCHAR2(80) := 'p_E7_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
  
  consum NUMBER(10,2);


  err_num NUMBER;
	err_msg VARCHAR2(255);
  
  CURSOR c_contractes IS
    select avg(consum)
    from consum;
   
BEGIN

	resultat := 'OK';
	entrada := '< >' ;
   
      -- tractament dels cursors
      delete from e7;
      
	OPEN c_contractes;
	LOOP
		FETCH c_contractes INTO consum;	
		EXIT WHEN c_contractes%NOTFOUND; 
			BEGIN
				INSERT INTO E7(consumMig) VALUES (consum);    
			END;
	END LOOP;
	CLOSE c_contractes;
      
 
    -- Execuci correcte del procediment
    p_log_insert(procediment, entrada, sortida, resultat);
EXCEPTION     
	WHEN OTHERS THEN
		err_num := SQLCODE;
		err_msg := SQLERRM;
		resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
		p_log_insert(procediment, entrada, sortida, resultat);   
END p_E7_insert;
/


