-- GESTIO ADRECES (ALTA-BAIXA-MODIFICACIO) --

----------------------------------------------
-- PAIS - alta
-- Parmetres entrada: idPais, nomPais
-- Parmetres sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------
CREATE OR REPLACE PROCEDURE p_pais_insert (
	PidPais IN pais.idPais%TYPE,
	PnomPais IN pais.nomPais%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_pais_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	noPais EXCEPTION;
	noNomPais EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
BEGIN
	-- Inicialitzacio de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' ||PnomPais || '>';
  sortida := ' ';

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN 
		RAISE noPais;
	END IF;
	IF (PnomPais IS NULL OR PnomPais = '') THEN
		RAISE noNomPais;
	END IF;

	-- Inserci a la taula
	INSERT INTO pais(idPais,nomPais) 
		VALUES (PidPais,PnomPais);

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

	-- Control d'excepcions
	EXCEPTION
		WHEN noPais THEN
			resultat := 'ERROR: <idPais> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noNomPais THEN
			resultat := 'ERROR <nomPais> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN DUP_VAL_ON_INDEX THEN
			resultat := 'ERROR: Pais ja existeix a la BD';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN OTHERS THEN
			err_num := SQLCODE;
			err_msg := SQLERRM;
			resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
			p_log_insert(procediment, entrada, sortida, resultat);
END p_pais_insert;
/

------------------------------------------------------
-- PAIS - update
-- Parmetres d'entrada: idPais, nomPais
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_pais_update (
	PidPais IN pais.idPais%TYPE, 
	PnomPais IN pais.nomPais%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_pais_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(100);

	noPais EXCEPTION;
	noNomPais EXCEPTION;
	noTrobatPais EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);

BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PnomPais || '>';
	
	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
		RAISE noPais;
	END IF;
 
	IF (PnomPais IS NULL OR PnomPais = '') THEN
		RAISE noNomPais;
	END IF;

	-- Comprova si el pas existeix
	BEGIN 
		SELECT nomPais INTO nom FROM pais
			WHERE PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 
	
	-- Update
	UPDATE pais 
		SET nomPais = PnomPais
	WHERE idPais = PidPais;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

	-- Control d'excepcions
	EXCEPTION
		WHEN noPais THEN
			resultat := 'ERROR: <idPais> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noNomPais THEN
			resultat := 'ERROR: <nomPas> inexistent';				
			p_log_insert(procediment, entrada, sortida, resultat);				
		WHEN noTrobatPais THEN
			resultat := 'ERROR: El pas no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN OTHERS THEN
			err_num := SQLCODE;
			err_msg := SQLERRM;
			resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
			p_log_insert(procediment, entrada, sortida, resultat);
END p_pais_update;
/

------------------------------------------------------
-- PAIS - delete
-- Parmetres d'entrada: idPais
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_pais_delete (
	PidPais IN pais.idPais%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_pais_delete';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);

	noTrobatPais EXCEPTION;
	noPais EXCEPTION;
	dependents EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '>';
		        
	-- Comprobaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
		RAISE noPais;
	END IF;

	BEGIN 
		SELECT nomPais INTO nom FROM Pais
			WHERE PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 
	
	-- Eliminaci
	BEGIN 
		DELETE FROM pais WHERE idPais = PidPais;
	EXCEPTION 
		WHEN OTHERS THEN
			RAISE dependents;
	END;
			

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

	-- Control d'excepcions
	EXCEPTION
		WHEN noPais THEN
			resultat := 'ERROR: <idPas> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatPais THEN
			resultat := 'ERROR: El pas no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN dependents THEN
			resultat := 'ERROR: El pas t localitats que en depenen i no pot ser eliminat de la BD';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN OTHERS THEN
			err_num := SQLCODE;
			err_msg := SQLERRM;
			resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
			p_log_insert(procediment, entrada, sortida, resultat);
END p_pais_delete;
/

------------------------------------------------------
-- LOCALITAT - insert
-- Parmetres d'entrada: idPais, nomLocalitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_localitat_insert (
	PidPais IN localitat.idPais%TYPE, 
	PnomLocalitat IN localitat.nomLocalitat%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_localitat_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);

	noPais EXCEPTION;
	noTrobatPais EXCEPTION;
	noNomLoc EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PnomLocalitat || '>';

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;
        
	IF (PnomLocalitat IS NULL OR PnomLocalitat = '') THEN
            RAISE noNomLoc;
	END IF;
		
	-- Comprovaci que existeixi el Pais
	BEGIN 
		SELECT nomPais INTO nom FROM Pais
            WHERE PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 

	-- Inserci a la taula
	INSERT INTO Localitat (idPais, nomLocalitat)
		VALUES (PidPais, PnomLocalitat);
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNomLoc THEN
				resultat := 'ERROR: <nomLocalitat> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobatPais THEN
				resultat := 'ERROR: El Pais no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: La Localitat en aquest pais ja existeix a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_localitat_insert;
/

------------------------------------------------------
-- LOCALITAT - update
-- Parmetres d'entrada:  idLocalitat, idPais, nomLocalitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_localitat_update (
	PidLocalitat IN localitat.idLocalitat%TYPE, 
	PidPais IN localitat.idPais%TYPE, 	
	PnomLocalitat IN localitat.nomLocalitat%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_localitat_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(100);

	noLoc EXCEPTION;
	noNomLoc EXCEPTION;
	noPais EXCEPTION;	
	noTrobatLoc EXCEPTION;
	noTrobatPais EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);

BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PnomLocalitat || '>';
	
	-- Comprovaci parmetres entrada
	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
		RAISE noLoc;
	END IF;
        
	IF (PidPais IS NULL OR PidPais = '') THEN
		RAISE noPais;
	END IF;
        
	IF (PnomLocalitat IS NULL OR PnomLocalitat = '') THEN
		RAISE noNomLoc;
	END IF;

	-- Comprovaci que existeixi la localitat
	BEGIN 
		SELECT nomLocalitat INTO nom FROM Localitat
			WHERE PidLocalitat = idLocalitat and idPais=PidPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatLoc;
	END; 

	-- Comprovaci que existeixi el pais
	BEGIN 
		SELECT nomPais INTO nom FROM Pais
			WHERE PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 
	
	-- Update
	UPDATE localitat SET	
		nomLocalitat = PnomLocalitat
	WHERE idLocalitat = PidLocalitat and idPais = PidPais;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

	-- Control d'excepcions
	EXCEPTION
		WHEN noLoc THEN
			resultat := 'ERROR: <idLocalitat> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noNomLoc THEN
			resultat := 'ERROR: <nomLocalitat> inexistent';				
			p_log_insert(procediment, entrada, sortida, resultat);				
		WHEN noPais THEN
			resultat := 'ERROR: <idPais> inexistent';				
			p_log_insert(procediment, entrada, sortida, resultat);				
		WHEN noTrobatLoc THEN
			resultat := 'ERROR: La localitat no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatPais THEN
			resultat := 'ERROR: El Pais no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN OTHERS THEN
			err_num := SQLCODE;
			err_msg := SQLERRM;
			resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
			p_log_insert(procediment, entrada, sortida, resultat);
END p_localitat_update;
/

------------------------------------------------------
-- LOCALITAT - delete
-- Parmetres d'entrada: idPais, idLocalitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_localitat_delete (
	PidLocalitat IN localitat.idLocalitat%TYPE,
	PidPais IN localitat.idPais%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_localitat_delete';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);

	noTrobatPais EXCEPTION;
	noTrobatLoc EXCEPTION;
	noPais EXCEPTION;
	noLoc EXCEPTION;
	dependents EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '>'; 
		        
	-- Comprobaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
		RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
		RAISE noLoc;
	END IF;
		

	-- Comprovem que existeixi el Pais
	BEGIN 
		SELECT nomPais INTO nom FROM pais
            WHERE PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 

	-- Comprovem que existeixi la Localitat
	BEGIN 
		SELECT nomLocalitat INTO nom FROM localitat
            WHERE PidLocalitat = idLocalitat AND PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatLoc;
	END; 
	
	-- Eliminaci
	BEGIN
		DELETE FROM localitat
			WHERE idLocalitat = PidLocalitat AND idPais=PidPais;
	EXCEPTION
		WHEN OTHERS THEN
			RAISE dependents;
	END;

	-- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

	-- Control d'excepcions
	EXCEPTION
		WHEN noPais THEN
			resultat := 'ERROR: <idPais> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatPais THEN
			resultat := 'ERROR: El pais no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noLoc THEN
			resultat := 'ERROR: <idLocalitat> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatLoc THEN
			resultat := 'ERROR: La localitat no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN dependents THEN
			resultat := 'ERROR: La localitat t adreces que en depenen i no pot ser eliminada de la BD';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN OTHERS THEN
			err_num := SQLCODE;
			err_msg := SQLERRM;
			resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
			p_log_insert(procediment, entrada, sortida, resultat);
END p_localitat_delete;
/



------------------------------------------------------
-- ADRECA - insert
-- Parmetres d'entrada: idPais, idLocalitat, codiPostal, carrer, num, escala, pis, porta
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_adreca_insert (
      PidPais IN adreca.idPais%TYPE,
	PidLocalitat IN adreca.idLocalitat%TYPE, 
	PcodiPostal IN adreca.codiPostal%TYPE,
	Pcarrer IN adreca.carrer%TYPE,
	Pnum IN adreca.num%TYPE,
	Pescala IN adreca.escala%TYPE,
	Ppis IN adreca.pis%TYPE,
	Pporta IN adreca.porta%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_adreca_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);
  
	noLoc EXCEPTION;
	noPais EXCEPTION;
	noTrobatLoc EXCEPTION;
	noTrobatPais EXCEPTION;
	
	noCodiPost EXCEPTION;
	noCarrer EXCEPTION;
	noNum EXCEPTION;
	noPis EXCEPTION;
	noPorta EXCEPTION;
	
  err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PcodiPostal || '> <' || Pcarrer || '> <' 
			|| Pnum || '> <' || Pescala || '> <' || Ppis || '> <' || Pporta || '>' ;

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PcodiPostal IS NULL OR PcodiPostal = '') THEN
            RAISE noCodiPost;
	END IF;

	IF (Pcarrer IS NULL OR Pcarrer = '') THEN
            RAISE noCarrer;
	END IF;

	IF (Pnum IS NULL OR Pnum = '') THEN
            RAISE noNum;
	END IF;

	IF (Ppis IS NULL OR Ppis = '') THEN
            RAISE noPis;
	END IF;	

	IF (Pporta IS NULL OR Pporta = '') THEN
            RAISE noPorta;
	END IF;

	-- Comprovaci que existeixi el Pais
	BEGIN 
		SELECT nomPais INTO nom FROM Pais
            WHERE PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 

	-- Comprovaci que existeixi la localitat
	BEGIN 
		SELECT nomLocalitat INTO nom FROM Localitat
            WHERE PidLocalitat = idLocalitat AND PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatLoc;
	END; 

	-- Inserci a la taula
	INSERT INTO Adreca(idPais, idLocalitat, codiPostal, carrer, num, escala, pis, porta)
		VALUES (PidPais, PidLocalitat, PcodiPostal, Pcarrer, Pnum, Pescala, Ppis, Pporta);
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCodiPost THEN
				resultat := 'ERROR: <codiPostal> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCarrer THEN
				resultat := 'ERROR: <carrer> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNum THEN
				resultat := 'ERROR: <num> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPis THEN
				resultat := 'ERROR: <pis> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPorta THEN
				resultat := 'ERROR: <porta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobatPais THEN
				resultat := 'ERROR: El pais no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLoc THEN
				resultat := 'ERROR: La localitat no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Adrea per aquesta localitat en aquest pais ja existeix a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_adreca_insert;
/

------------------------------------------------------
-- ADRECA - update
-- Parmetres d'entrada: idPais, idLocalitat, idAdreca, codiPostal, carrer, num, escala, pis, porta
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_adreca_update (
  PidPais IN adreca.idPais%TYPE,
	PidLocalitat IN adreca.idLocalitat%TYPE, 
  PidAdreca IN adreca.idAdreca%TYPE,
	PcodiPostal IN adreca.codiPostal%TYPE,
	Pcarrer IN adreca.carrer%TYPE,
	Pnum IN adreca.num%TYPE,
	Pescala IN adreca.escala%TYPE,
	Ppis IN adreca.pis%TYPE,
	Pporta IN adreca.porta%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_adreca_udpate';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);
  
	noPais EXCEPTION;
	noLoc EXCEPTION;
  noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noTrobatLoc EXCEPTION;
	noTrobatPais EXCEPTION;
	
	noCodiPost EXCEPTION;
	noCarrer EXCEPTION;
	noNum EXCEPTION;
	noPis EXCEPTION;
	noPorta EXCEPTION;
	
  err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PcodiPostal || '> <' || Pcarrer || '> <' 
			|| Pnum || '> <' || Pescala || '> <' || Ppis || '> <' || Pporta || '>' ;

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PcodiPostal IS NULL OR PcodiPostal = '') THEN
            RAISE noCodiPost;
	END IF;

	IF (Pcarrer IS NULL OR Pcarrer = '') THEN
            RAISE noCarrer;
	END IF;

	IF (Pnum IS NULL OR Pnum = '') THEN
            RAISE noNum;
	END IF;

	IF (Ppis IS NULL OR Ppis = '') THEN
            RAISE noPis;
	END IF;	

	IF (Pporta IS NULL OR Pporta = '') THEN
            RAISE noPorta;
	END IF;

	-- Comprovaci que existeixi el Pais
	BEGIN 
		SELECT nomPais INTO nom FROM Pais
            WHERE PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 

	-- Comprovaci que existeixi la localitat
	BEGIN 
		SELECT nomLocalitat INTO nom FROM Localitat
            WHERE PidLocalitat = idLocalitat AND PidPais = idPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatLoc;
	END; 

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM adreca
            WHERE PidLocalitat = idLocalitat AND PidPais = idPais AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Update
	UPDATE adreca SET
		codiPostal=PcodiPostal,
		carrer=Pcarrer,
		num=Pnum,
		escala=Pescala,
		pis=Ppis,
		porta=Pporta
	WHERE idLocalitat = PidLocalitat AND idPais = PidPais AND idAdreca = PidAdreca;
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCodiPost THEN
				resultat := 'ERROR: <codiPostal> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCarrer THEN
				resultat := 'ERROR: <carrer> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNum THEN
				resultat := 'ERROR: <num> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPis THEN
				resultat := 'ERROR: <pis> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPorta THEN
				resultat := 'ERROR: <porta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobatPais THEN
				resultat := 'ERROR: El pais no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLoc THEN
				resultat := 'ERROR: La localitat no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: La adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_adreca_update;
/

------------------------------------------------------
-- ADRECA - delete
-- Parmetres d'entrada: idPais,idLocalitat,idAdreca
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_adreca_delete (
	PidPais IN adreca.idPais%TYPE,
	PidLocalitat IN adreca.idLocalitat%TYPE,
	PidAdreca IN adreca.idAdreca%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_adreca_delete';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	nom VARCHAR2(50);
	nomLoc VARCHAR2(120);
  nomCarrer VARCHAR2(255);
	noPais EXCEPTION;
	noLoc EXCEPTION;
  noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noTrobatLoc EXCEPTION;
	noTrobatPais EXCEPTION;
	dependents EXCEPTION;
  err_num NUMBER;
	err_msg VARCHAR2(255);
	
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '>';
		        
	-- Comprobaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	-- Comprovaci que existeixi el Pais
	BEGIN 
		SELECT nomPais INTO nom FROM Pais
            WHERE idPais = PidPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatPais;
	END; 

	-- Comprovaci que existeixi la localitat
	BEGIN 
		SELECT nomLocalitat INTO nomLoc FROM Localitat
            WHERE idLocalitat = PidLocalitat AND idPais = PidPais;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatLoc;
	END; 

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nomCarrer FROM adreca
            WHERE idLocalitat = PidLocalitat AND idPais = PidPais AND idAdreca=PidAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 
	
	-- Eliminaci
	BEGIN
		DELETE FROM adreca
			WHERE idAdreca=PidAdreca AND idLocalitat = PidLocalitat AND idPais=PidPais;
	EXCEPTION
		WHEN OTHERS THEN
			RAISE dependents;
	END;
	-- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

	-- Control d'excepcions
	EXCEPTION
		WHEN noPais THEN
			resultat := 'ERROR: <idPais> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noLoc THEN
			resultat := 'ERROR: <idLocalitat> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noAdreca THEN
			resultat := 'ERROR: <idAdreca> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatPais THEN
			resultat := 'ERROR: El pais no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatLoc THEN
			resultat := 'ERROR: La localitat no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatAdreca THEN
			resultat := 'ERROR: La adreca no existeix';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN dependents THEN
			resultat := 'ERROR: Adrea t dependncies i no pot ser eliminada de la BD';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_adreca_delete;
/
