-- GESTIO CLIENTS (ALTA-BAIXA-MODIFICACIO) --

------------------------------------------------------
-- CLIENT - insert
-- Parmetres d'entrada: DNI, nomClient, idPais, idLocalitat, idAdreca)
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_client_insert (
  Pdni IN client.DNI%TYPE,
	PnomClient IN client.nomClient%TYPE, 
	PidPais IN client.idPais%TYPE,
	PidLocalitat IN client.idLocalitat%TYPE,
	PidAdreca IN client.idAdreca%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_client_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	DNI VARCHAR(10);
	nomClient VARCHAR2(255);
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;

	noDNI EXCEPTION;
	noNomClient EXCEPTION;
  noPais EXCEPTION;
	noLoc EXCEPTION;
  noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PDNI || '> <' || PnomClient || '> <' || PidPais || '> <' || PidLocalitat|| '> <' 
			|| PidAdreca|| '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (Pdni IS NULL OR Pdni = '') THEN
            RAISE noDNI;
	END IF;

	IF (PnomClient IS NULL OR PnomClient = '') THEN
            RAISE noNomClient;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Inserci a la taula (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO client(DNI, nomClient, idPais, idLocalitat, idAdreca, estat, dataEstat)
		VALUES (Pdni, PnomClient, PidPais, PidLocalitat, PidAdreca, 'A', campdataalta);
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noDNI THEN
				resultat := 'ERROR: <DNI> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNomClient THEN
				resultat := 'ERROR: <nomClient> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Client ja est actiua a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_client_insert;
/

------------------------------------------------------
-- CLIENT - update
-- Parmetres d'entrada: DNI, nomClient, idPais, idLocalitat, idAdreca)
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_client_update (
  Pdni IN client.DNI%TYPE,
	PnomClient IN client.nomClient%TYPE, 
	PidPais IN client.idPais%TYPE,
	PidLocalitat IN client.idLocalitat%TYPE,
	PidAdreca IN client.idAdreca%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_client_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE;
	Pestat_actual CHAR(1);

	DNI VARCHAR(10);
	nomClient VARCHAR2(255);
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	estat CHAR(1);
	nom VARCHAR2(50);

	noDNI EXCEPTION;
	noNomClient EXCEPTION;
 	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noTrobatClient EXCEPTION;
	noAlta EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PDNI || '> <' || PnomClient || '> <' || PidPais || '> <' || PidLocalitat|| '> <' 
			|| PidAdreca|| '>';
	campdatamodifica := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (Pdni IS NULL OR Pdni = '') THEN
            RAISE noDNI;
	END IF;

	IF (PnomClient IS NULL OR PnomClient = '') THEN
            RAISE noNomClient;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	-- Comprovaci que existeixi el client
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Client
            WHERE Pdni=DNI;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatClient;
	END; 

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 
   
	-- Si el client est donat de baixa cal donar-l'ho d'alta prviament
	IF (Pestat_actual = 'A') THEN
	-- Update
		UPDATE client SET
			nomClient=PnomClient,
			idPais=PidPais,
			idLocalitat=PidLocalitat,
			idAdreca=PidAdreca
		WHERE DNI = Pdni;
	ELSE
		RAISE noAlta;
	END IF;


	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noDNI THEN
				resultat := 'ERROR: <DNI> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNomClient THEN
				resultat := 'ERROR: <nomClient> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatClient THEN
				resultat := 'ERROR: Client no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAlta THEN
				resultat := 'ERROR: Client donat de baixa, cal donar-la d alta previament';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_client_update;
/

------------------------------------------------------
-- CLIENT - canvia l'estat d'un client
-- Parmetres d'entrada: DNI, estat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_client_canviestat (
      Pdni IN client.DNI%TYPE,
      Pestat IN client.Estat%TYPE,
      resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_client_canviestat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE;
	Pestat_actual CHAR(1);

	DNI VARCHAR(10);
	estat CHAR(1);

	noDNI EXCEPTION;
	noEstat EXCEPTION;
	noTrobatClient EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PDNI || '> <' || Pestat || '>' ;
	campdatamodifica := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (Pdni IS NULL OR Pdni = '') THEN
            RAISE noDNI;
	END IF;

	IF (Pestat <> 'A' AND Pestat <> 'B') THEN
            RAISE noEstat;
	END IF;

	-- Comprovaci que existeixi el client
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Client
            WHERE Pdni=DNI;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatClient;
	END; 

	-- Update estat del client
	UPDATE client SET
		estat=Pestat,
		dataEstat=campdatamodifica
	WHERE DNI = Pdni;

	-- es canvia, tamb, l'estat dels comptadors associats al client
	UPDATE comptador SET
			estat=Pestat,
			dataEstat=campdatamodifica
	WHERE DNI = PDNI;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noDNI THEN
				resultat := 'ERROR: <DNI> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEstat THEN
				resultat := 'ERROR: <Estat> inexistent o incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatClient THEN
				resultat := 'ERROR: Client no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_client_canviestat;
/

