-- GESTIO COMPTADORS (ALTA-BAIXA-MODIFICACIO) --

------------------------------------------------------
-- COMPTADOR - insert
-- Parmetres d'entrada: potenciaContractada, modelComptador, empresaFabricacio, dataFabricacio, idPais, idLocalitat, idAdreca, DNI
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Comptador_insert (
	PpotenciaContractada IN comptador.potenciacontractada%TYPE,
	PmodelComptador IN comptador.modelComptador%TYPE,
	PempresaFabricacio IN comptador.empresaFabricacio%TYPE,
	PdataFabricacio IN comptador.dataFabricacio%TYPE,
	PidPais IN comptador.idPais%TYPE,
	PidLocalitat IN comptador.idLocalitat%TYPE,
	PidAdreca IN comptador.idAdreca%TYPE,
	PDNI IN comptador.DNI%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_comptador_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(100);
	estat_client CHAR(1);
	DNI VARCHAR(10);
  
	noPotencia EXCEPTION;
	noModel EXCEPTION;
	noEmpresa EXCEPTION;
	noFabricacio EXCEPTION;
	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noDNI EXCEPTION;
	noTrobatClient EXCEPTION;
	ClientNoActiu EXCEPTION;
	
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';

	entrada := '<' || PpotenciaContractada || '> <' || PmodelComptador || '> <' || PempresaFabricacio || '> <' 
			|| PdataFabricacio || '> <' || PidPais || '> <' || PidLocalitat|| '> <' || PidAdreca || '> <' || PDNI || '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (PpotenciaContractada IS NULL OR PpotenciaContractada = 0.0) THEN
            RAISE noPotencia;
	END IF;
	IF (PmodelComptador IS NULL OR PmodelComptador = '') THEN
            RAISE noModel;
	END IF;
	IF (PempresaFabricacio IS NULL OR PempresaFabricacio  = '') THEN
		RAISE noEmpresa ;
	END IF;
	IF (PdataFabricacio IS NULL OR PdataFabricacio = '') THEN
		RAISE noFabricacio;
	END IF;
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;
	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;
	IF (PDNI IS NULL OR PDNI = '') THEN
            RAISE noDNI;
	END IF;
	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatAdreca;
	END; 

	-- Comprovaci que existeixi el client
	BEGIN 
		SELECT estat INTO estat_client FROM Client
            WHERE PDNI = DNI ;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatClient;
	END; 
  
	-- Inserci a la taula (per defecte s'assigna com a estat 'A' = actiu a la BD)
	IF (estat_client <> 'B') THEN
	INSERT INTO comptador(potenciaContractada, modelComptador, empresaFabricacio, dataFabricacio,
                        estat, dataEstat, idPais, idLocalitat, idAdreca, DNI )
		VALUES (PpotenciaContractada, PmodelComptador, PempresaFabricacio, PdataFabricacio,
            'A', campdataalta, PidPais, PidLocalitat, PidAdreca,PDNI);

-- Execuci correcta del procediment
		p_log_insert(procediment, entrada, sortida, resultat);
	ELSE
		RAISE ClientNoActiu;
	END IF;
	-- Control d'excepcions
	EXCEPTION
			WHEN noPotencia THEN
				resultat := 'ERROR: <PotenciaContractada> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noModel THEN
				resultat := 'ERROR: <ModelComptador> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEmpresa  THEN
				resultat := 'ERROR: <EmpresaFabricacio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noFabricacio  THEN
				resultat := 'ERROR: <DataFabricacio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDNI THEN
				resultat := 'ERROR: <DNI> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);        
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatClient THEN
				resultat := 'ERROR: Client no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);        
			WHEN ClientNoActiu THEN
				resultat := 'ERROR: Client no est actiua la BD';
				p_log_insert(procediment, entrada, sortida, resultat);   
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Comptador ja est actiu a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Comptador_insert;
/

------------------------------------------------------
-- COMPTADOR - update
-- Parmetres d'entrada: codiContracte, potenciaContractada, modelComptador, empresaFabricacio, dataFabricacio, idPais, idLocalitat, idAdreca, DNI
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Comptador_update (
  PcodiContracte IN comptador.codiContracte%TYPE,
  PpotenciaContractada IN comptador.potenciacontractada%TYPE,
  PmodelComptador IN comptador.modelComptador%TYPE,
  PempresaFabricacio IN comptador.empresaFabricacio%TYPE,
  PdataFabricacio IN comptador.dataFabricacio%TYPE,
  PidPais IN comptador.idPais%TYPE,
  PidLocalitat IN comptador.idLocalitat%TYPE,
  PidAdreca IN comptador.idAdreca%TYPE,
  PDNI IN comptador.DNI%TYPE,
  resultat OUT NOCOPY VARCHAR2
)
IS
	procediment VARCHAR2(80) := 'p_comptador_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	
  nom VARCHAR(50);
	estatClient CHAR(1);
	DNI VARCHAR(10);
	Pestat_actual CHAR(1);


	noCodiContracte EXCEPTION;
	noPotencia EXCEPTION;
	noModel EXCEPTION;
	noEmpresa EXCEPTION;
	noFabricacio EXCEPTION;
	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
 	noDNI EXCEPTION;
	noTrobatClient EXCEPTION;
	NoTrobatContracte EXCEPTION;
	ComptadorNoActiu EXCEPTION;
	ClientNoActiu EXCEPTION;


	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';

	entrada := '<' || PcodiContracte || '> <' || PpotenciaContractada || '> <' || PmodelComptador || '> <' || PempresaFabricacio || '> <' 
			|| PdataFabricacio || '> <' || PidPais || '> <' || PidLocalitat|| '> <' || PidAdreca || '> <' || PDNI || '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada
	IF (PcodiContracte IS NULL OR PcodiContracte = 0) THEN
            RAISE noCodiContracte;
	END IF;
	IF (PpotenciaContractada IS NULL OR PpotenciaContractada = 0.0) THEN
            RAISE noPotencia;
	END IF;
	IF (PmodelComptador IS NULL OR PmodelComptador = '') THEN
            RAISE noModel;
	END IF;
  	IF (PempresaFabricacio IS NULL OR PempresaFabricacio  = '') THEN
            RAISE noEmpresa ;
	END IF;
	 IF (PdataFabricacio IS NULL OR PdataFabricacio = '') THEN
            RAISE noFabricacio;
	END IF;
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;
	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;
	IF (PDNI IS NULL OR PDNI = '') THEN
            RAISE noDNI;
	END IF;

	-- Comprovaci que existeixi el Contracte
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Comptador
		WHERE PcodiContracte = codiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN 
			RAISE noTrobatContracte;
	END;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatAdreca;
	END; 

	-- Comprovaci que existeixi el client
	BEGIN 
		SELECT estat INTO estatClient FROM Client
            WHERE PDNI = DNI ;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatClient;
	END; 
	
	-- Comprovaci que el client estigui actiu
	IF (estatClient <> 'A') THEN
		RAISE ClientNoActiu;
	END IF;
		
	

	-- Si el comptador est actiu s'actualitza la informaci

	IF (Pestat_actual = 'A') THEN
		UPDATE Comptador SET
			potenciaContractada = PpotenciaContractada,
			modelComptador = PmodelComptador,
			empresaFabricacio = PempresaFabricacio,
			dataFabricacio = PdataFabricacio,
			idPais = PidPais,
			idLocalitat=PidLocalitat,
			idAdreca = PidAdreca,
			DNI=PDNI
		WHERE codiContracte = PcodiContracte;
	ELSE
		RAISE ComptadorNoActiu;
	END IF;
			
-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiContracte THEN
				resultat := 'ERROR: <CodiContracte> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPotencia THEN
				resultat := 'ERROR: <PotenciaContractada> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noModel THEN
				resultat := 'ERROR: <ModelComptador> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEmpresa  THEN
				resultat := 'ERROR: <EmpresaFabricacio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noFabricacio  THEN
				resultat := 'ERROR: <DataFabricacio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDNI THEN
				resultat := 'ERROR: <DNI> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);        
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatClient THEN
				resultat := 'ERROR: Client no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);        
			WHEN noTrobatContracte THEN
				resultat := 'ERROR: No s ha trobat el comptador a la base de dades';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN ComptadorNoActiu THEN
				resultat := 'ERROR: El comptador est donat de baixa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN ClientNoActiu THEN
				resultat := 'ERROR: El client est donat de baixa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Comptador_update;
/

------------------------------------------------------
-- COMPTADOR - canviEstat
-- Parmetres d'entrada: codiContracte
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_comptador_canviEstat (
      PcodiContracte IN comptador.codiContracte%TYPE,
      Pestat IN comptador.estat%TYPE,
      resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_comptador_canviEstat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE;
	Pestat_actual CHAR(1);

	codiContracte INTEGER;
	estat CHAR(1);

	noCodiContracte EXCEPTION;
	noEstat EXCEPTION;
	noTrobatContracte EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiContracte || '> <' || Pestat || '>' ;
	campdatamodifica := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (PcodiContracte IS NULL OR PcodiContracte = 0) THEN
            RAISE nocodiContracte;
	END IF;

	IF (Pestat <> 'A' AND Pestat <> 'B') THEN
            RAISE noEstat;
	END IF;

	-- Comprovaci que existeixi el comptador
	BEGIN 
		SELECT estat INTO Pestat_actual FROM comptador
            WHERE PcodiContracte=codiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatContracte;
	END; 

	-- Update
	UPDATE comptador SET
		estat=Pestat,
		dataEstat=campdatamodifica
	WHERE codiContracte = PcodiContracte;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiContracte THEN
				resultat := 'ERROR: <codiContracte> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEstat THEN
				resultat := 'ERROR: <Estat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatContracte THEN
				resultat := 'ERROR: Comptador no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_comptador_canviEstat;
/

------------------------------------------------------
-- CONSUM - insert consum (lecturapresencial)
-- Parmetres d'entrada: codiContracte, dataLectura, consum
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_ConsumP_insert (
  PcodiContracte IN consum.codiContracte%TYPE,
  PdataLectura IN consum.dataLectura%TYPE,
  Pconsum IN consum.consum%TYPE,
  resultat OUT NOCOPY VARCHAR2
)
IS
	procediment VARCHAR2(80) := 'p_consumP_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE ;
  
	codiContracte INTEGER;
	estat CHAR(1);
	Pestat_actual CHAR(1);
	Linia INTEGER;
	potencia NUMBER(10,2);
	consumComptadors NUMBER(10,2);
	energiaCentrals NUMBER(10,2);

	noCodiContracte EXCEPTION;
	nodataLectura EXCEPTION;
	noConsum EXCEPTION;
	noTrobatContracte EXCEPTION;
	ComptadorNoActiu EXCEPTION;
	ComptadorNoConnectat EXCEPTION;
	errorLectura EXCEPTION;
	noEnergia EXCEPTION;
  
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiContracte || '> <' || PdataLectura  || '> <' || Pconsum || '>' ;
	campdatamodifica := SYSDATE;
  

	-- Comprovaci parmetres entrada
	IF (PcodiContracte IS NULL OR PcodiContracte = 0) THEN
            RAISE nocodiContracte;
	END IF;

	IF (PdataLectura IS NULL OR PdataLectura = '') THEN
            RAISE nodataLectura;
	END IF;

	IF (PConsum IS NULL OR PConsum = 0.0) THEN
            RAISE noConsum;
	END IF;

	-- Comprovaci que existeixi el comptador
	BEGIN 
		SELECT estat INTO Pestat_actual FROM comptador
            WHERE PcodiContracte=codiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatContracte;
	END; 

  -- comprovem que el comptador estigui connectat a una centraleta
	BEGIN 
		SELECT COUNT(DISTINCT cn.idLinia) INTO Linia FROM connecta cn
		INNER JOIN LiniesComunicacio lc
			ON cn.idLinia=lc.idLinia AND lc.estat='A'
	      WHERE cn.codiContracte=PcodiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE ComptadorNoConnectat;
	END;

    -- comprovem que el comptador estigui connectat a una centraleta
	BEGIN 
		SELECT COUNT(DISTINCT al.idLinia) INTO Linia FROM alimenta al
		INNER JOIN connecta cn
			ON al.codiCentraleta=cn.codiCentraleta AND al.idPais = cn.idPais AND al.idLocalitat=cn.idLocalitat
			AND al.idAdreca=cn.idADRECA
		INNER JOIN centraletaDistribucio cd
			ON al.codiCentraleta=cd.codiCentraleta AND al.idPais = cd.idPais AND al.idLocalitat=cd.idLocalitat
			AND al.idAdreca=cd.idADRECA and cd.estat='A'
		INNER JOIN LiniesComunicacio lc
			ON al.idLinia=lc.idLinia AND lc.estat='A'
		INNER JOIN CentralProduccio cp
			ON al.codiCentral=cp.CodiCentral AND cp.estat='A'
	      WHERE cn.codiContracte=PcodiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE ComptadorNoConnectat;
	END;
  

  -- comprovem que el consum no superi la potencia contractada (si supera s que hi ha un error de lectura)
	BEGIN 
		SELECT potenciaContractada INTO potencia FROM comptador
		WHERE PcodiContracte=codiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatContracte;
	END; 

	IF (Pconsum > potencia) THEN
		RAISE errorLectura;
	END IF;
  
  
  -- Energia produda fins la data de lectura per les centrals que alimenten el comptador
	select sum(p.produccioMes) into energiaCentrals
	from produccio p
	where p.codiCentral IN (select distinct cp.codiCentral
					from centralProduccio cp 
					inner join alimenta al
						on al.codiCentral=cp.codiCentral
					inner join connecta cn
						on al.codiCentraleta = cn.codiCentraleta 
							and al.idPais = cn.idPais and al.idLocalitat = cn.idLocalitat 
							and al.idAdreca = cn.idAdreca
					where cn.codiContracte = PcodiContracte) 
		and p.dataLectura < campdatamodifica ;  
  
  
-- Consum dels comptadors de les centrals de produccio que alimenten un comptador determinat abans d'una data determinada
	select sum(cs.consum) into consumComptadors
	from consum cs 
	where cs.dataLectura < campdatamodifica  
      	and cs.codiContracte IN (
	        select distinct cn.codiContracte
	        from connecta cn
	        inner join alimenta al
	              on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
	                and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
	              where al.codiCentral IN (
	                select distinct cp.codiCentral
	                from centralProduccio cp 
	                inner join alimenta al
	                  on al.codiCentral=cp.codiCentral
	                inner join connecta cn
	                  on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
	                     and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca
	                  where cn.codiContracte = PcodiContracte));
  

	IF (Pconsum > (energiaCentrals - consumComptadors) ) THEN
	  RAISE noEnergia;
	END IF;

	IF (Pestat_actual = 'A') THEN
	-- Insert
		INSERT INTO consum (codiContracte,dataLectura,consum,tipusLectura) VALUES (PcodiContracte,PdataLectura,Pconsum,'P');
	ELSE 	
		RAISE ComptadorNoActiu;
	END IF;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
	resultat:='Ok';
	p_e1_insert(PcodiContracte,Pconsum,resultat);
	resultat:='Ok';
	p_e2_insert(PdataLectura,PcodiContracte,Pconsum,resultat);  
	resultat:='Ok';
	p_e3_insert(resultat);        
	resultat:='Ok';
	p_e4_insert(PdataLectura,resultat);
	resultat:='Ok';
	p_e6_insert(resultat);  
	resultat:='Ok';
	p_e7_insert(resultat);   
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiContracte THEN
				resultat := 'ERROR: <codiContracte> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noConsum THEN
				resultat := 'ERROR: <Consum> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataLectura THEN
				resultat := 'ERROR: <dataLectura> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatContracte THEN
				resultat := 'ERROR: Comptador no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
      		WHEN ComptadorNoConnectat THEN
				resultat := 'ERROR: Comptador no esta connectat a la xarxa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEnergia THEN
				resultat := 'ERROR: Consum Erroni. No hi ha prou energia disponible per aquest consum';
				p_log_insert(procediment, entrada, sortida, resultat);        
			WHEN ComptadorNoActiu THEN
				resultat := 'ERROR: El comptador no est actiu a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN errorLectura THEN
				resultat := 'ERROR: El valor del consum introdut s superior a la potncia contractada';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Valor de la lectura ja registrat';
				p_log_insert(procediment, entrada, sortida, resultat);        
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_ConsumP_insert;
/


------------------------------------------------------
-- CONSUM - insert consum (lectura telematica)
-- Parmetres d'entrada: codiContracte, consum
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_ConsumT_insert (
  PcodiContracte IN consum.codiContracte%TYPE,
  Pconsum IN consum.consum%TYPE,
  resultat OUT NOCOPY VARCHAR2
)
IS
	procediment VARCHAR2(80) := 'p_consumT_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatainsert DATE ;
  
	codiContracte INTEGER;
	estat CHAR(1);
	Pestat_actual CHAR(1);
	Linia INTEGER;
	potencia NUMBER(10,2);
	consumComptadors NUMBER(10,2);
	energiaCentrals NUMBER(10,2);
  
  
	noCodiContracte EXCEPTION;
	noConsum EXCEPTION;
	noTrobatContracte EXCEPTION;
	ComptadorNoActiu EXCEPTION;
	ComptadorNoConnectat EXCEPTION;
	errorLectura EXCEPTION;
	noEnergia EXCEPTION;  
  
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiContracte || '> <' || Pconsum || '>' ;
	campdatainsert := SYSDATE;
  

	-- Comprovaci parmetres entrada

	IF (PcodiContracte IS NULL OR PcodiContracte = 0) THEN
            RAISE nocodiContracte;
	END IF;

	IF (PConsum IS NULL OR PConsum = 0.0) THEN
            RAISE noConsum;
	END IF;

	-- Comprovaci que existeixi el comptador
	BEGIN 
		SELECT estat INTO Pestat_actual FROM comptador
            WHERE PcodiContracte=codiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatContracte;
	END; 

  -- comprovem que el comptador estigui connectat a una centraleta
	BEGIN 
		SELECT COUNT(DISTINCT cn.idLinia) INTO Linia FROM connecta cn
		INNER JOIN LiniesComunicacio lc
			ON cn.idLinia=lc.idLinia AND lc.estat='A'
	      WHERE cn.codiContracte=PcodiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE ComptadorNoConnectat;
	END;

    -- comprovem que el comptador estigui connectat a una centraleta
	BEGIN 
		SELECT COUNT(DISTINCT al.idLinia) INTO Linia FROM alimenta al
		INNER JOIN connecta cn
			ON al.codiCentraleta=cn.codiCentraleta AND al.idPais = cn.idPais AND al.idLocalitat=cn.idLocalitat
			AND al.idAdreca=cn.idADRECA
		INNER JOIN centraletaDistribucio cd
			ON al.codiCentraleta=cd.codiCentraleta AND al.idPais = cd.idPais AND al.idLocalitat=cd.idLocalitat
			AND al.idAdreca=cd.idADRECA and cd.estat='A'
		INNER JOIN LiniesComunicacio lc
			ON al.idLinia=lc.idLinia AND lc.estat='A'
		INNER JOIN CentralProduccio cp
			ON al.codiCentral=cp.CodiCentral AND cp.estat='A'
	      WHERE cn.codiContracte=PcodiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE ComptadorNoConnectat;
	END;
  

  -- comprovem que el consum no superi la potencia contractada (si supera s que hi ha un error de lectura)
	BEGIN 
		SELECT potenciaContractada INTO potencia FROM comptador
            WHERE PcodiContracte=codiContracte;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatContracte;
	END; 

	IF(Pconsum > potencia) THEN
		RAISE errorLectura;
	END IF;
  
-- Energia produda fins la data de lectura per les centrals que alimenten el comptador
	select sum(p.produccioMes) into energiaCentrals
	from produccio p
	where p.codiCentral IN (select distinct cp.codiCentral
					from centralProduccio cp 
					inner join alimenta al
						on al.codiCentral=cp.codiCentral
					inner join connecta cn
						on al.codiCentraleta = cn.codiCentraleta and 
						al.idPais = cn.idPais and al.idLocalitat = cn.idLocalitat and 
						al.idAdreca = cn.idAdreca
					where cn.codiContracte = PcodiContracte)
		and p.dataLectura < campdatainsert ;  
  
  
-- Consum dels comptadors de les centrals de produccio que alimenten un comptador determinat abans d'una data determinada
	select sum(cs.consum) into consumComptadors
	from consum cs 
	where cs.dataLectura < campdatainsert  
	      and cs.codiContracte IN (
	        select distinct cn.codiContracte
	        from connecta cn
	        inner join alimenta al
	              on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
	                and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca 
	              where al.codiCentral IN (
	                select distinct cp.codiCentral
	                from centralProduccio cp 
	                inner join alimenta al
	                  on al.codiCentral=cp.codiCentral
	                inner join connecta cn
	                  on al.codiCentraleta = cn.codiCentraleta and al.idPais = cn.idPais 
	                     and al.idLocalitat = cn.idLocalitat and al.idAdreca = cn.idAdreca
	                  where cn.codiContracte = PcodiContracte));
  

	IF (Pconsum > (energiaCentrals - consumComptadors) ) THEN
		RAISE noEnergia;
	END IF;

	IF (Pestat_actual = 'A') THEN
	-- Insert
		INSERT INTO consum (codiContracte,dataLectura,consum,tipusLectura) VALUES (PcodiContracte,campdatainsert,Pconsum,'T');
  	ELSE 	
		RAISE ComptadorNoActiu;
	END IF;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);   
	resultat:='Ok';
	p_e1_insert(PcodiContracte,Pconsum,resultat);
	resultat:='Ok';
	p_e2_insert(campdatainsert,PcodiContracte,Pconsum,resultat);
	resultat:='Ok';
	p_e3_insert(resultat);
	resultat:='Ok';
	p_e4_insert(campdatainsert,resultat);
	resultat:='Ok';
	p_e6_insert(resultat);  
	resultat:='Ok';
	p_e7_insert(resultat);    
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiContracte THEN
				resultat := 'ERROR: <codiContracte> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noConsum THEN
				resultat := 'ERROR: <Consum> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEnergia THEN
				resultat := 'ERROR: Consum Erroni. No hi ha prou energia disponible per aquest consum';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatContracte THEN
				resultat := 'ERROR: Comptador no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
      		WHEN ComptadorNoConnectat THEN
				resultat := 'ERROR: Comptador no esta connectat a la xarxa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN ComptadorNoActiu THEN
				resultat := 'ERROR: El comptador no est actiu a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN errorLectura THEN
				resultat := 'ERROR: El valor del consum introdut s superior a la potncia contractada';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Lectura repetida';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_ConsumT_insert;
/




