-- GESTIO CENTRALS DE PRODUCCIO (ALTA-BAIXA-MODIFICACIO) --

------------------------------------------------------
-- CENTRAL PRODUCCIO NUCLEAR - insert
-- Parmetres d'entrada: idPais, idLocalitat, idAdreca, energiaMax, energiaMin, rebuig
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Nuclear_insert (
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	PenergiaMin IN nuclear.energiaMin%TYPE,
	Prebuig IN nuclear.rebuig%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Nuclear_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	energiaMin NUMBER(10,2);
	rebuig NUMBER(10,2);

  	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noEnergiaMin EXCEPTION;
	noRebuig EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PenergiaMin || '> <' || Prebuig || 	'>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (PenergiaMin IS NULL OR PenergiaMin = 0.0) THEN
            RAISE noEnergiaMin;
	END IF;

	IF (Prebuig IS NULL OR Prebuig = 0.0) THEN
            RAISE noRebuig;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Inserci a la taula de centrals de produccio (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO centralProduccio(idPais, idLocalitat, idAdreca, energiaMax, dataInspeccio, estat, dataEstat)
		VALUES (PidPais, PidLocalitat, PidAdreca, PenergiaMax, campdataalta, 'A', campdataalta);
      
	-- Inserci a la taula de centrals nuclears
	INSERT INTO nuclear(codiCentral, energiaMin, rebuig) VALUES
		(seq_central.currval, PenergiaMin, Prebuig);

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEnergiaMin THEN
				resultat := 'ERROR: <EnergiaMin> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noRebuig THEN
				resultat := 'ERROR: <Rebuig> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: La central nuclear ja est activa a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Nuclear_insert;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO HIDROELECTRICA - insert
-- Parmetres d'entrada: idPais, idLocalitat, idAdreca, energiaMax, turbines
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Hidroelectrica_insert (
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	Pturbines IN hidroelectrica.turbines%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Hidroelectrica_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	turbines INTEGER;

  	noPais EXCEPTION;
	noLoc EXCEPTION;
  	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noTurbines EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PTurbines || '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Pturbines IS NULL OR Pturbines = 0) THEN
            RAISE noTurbines;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Inserci a la taula de centrals de produccio (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO centralProduccio(idPais, idLocalitat, idAdreca, energiaMax, dataInspeccio, estat, dataEstat)
		VALUES (PidPais, PidLocalitat, PidAdreca, PenergiaMax, campdataalta, 'A', campdataalta);
      
	-- Inserci a la taula de centrals nuclears
	INSERT INTO hidroelectrica(codiCentral, turbines) VALUES
		(seq_central.currval, Pturbines);

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTurbines THEN
				resultat := 'ERROR: <Turbines> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Central Hidroelectrica ja est activa a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Hidroelectrica_insert;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO TERMICA - insert
-- Parmetres d'entrada: idPais, idLocalitat, idAdreca, energiaMax, emissions
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Termica_insert (
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	Pemissions IN termica.emissions%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Termica_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	emissions NUMBER(10,2);

  	noPais EXCEPTION;
	noLoc EXCEPTION;
  	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noEmissions EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PEmissions || '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Pemissions IS NULL OR Pemissions = 0.0) THEN
            RAISE noEmissions;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Inserci a la taula de centrals de produccio (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO centralProduccio(idPais, idLocalitat, idAdreca, energiaMax, dataInspeccio, estat, dataEstat)
		VALUES (PidPais, PidLocalitat, PidAdreca, PenergiaMax, campdataalta, 'A', campdataalta);
      
	-- Inserci a la taula de centrals nuclears
	INSERT INTO termica(codiCentral, emissions) VALUES
		(seq_central.currval, Pemissions);

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEmissions THEN
				resultat := 'ERROR: <Emissions> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Central Trmica ja est activa a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Termica_insert;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO EOLICA- insert
-- Parmetres d'entrada: idPais, idLocalitat, idAdreca, energiaMax, molins
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Eolica_insert (
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	Pmolins IN eolica.molins%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Eolica_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	molins INTEGER;

  	noPais EXCEPTION;
	noLoc EXCEPTION;
 	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noMolins EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || Pmolins || '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Pmolins IS NULL OR Pmolins = 0) THEN
            RAISE noMolins;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Inserci a la taula de centrals de produccio (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO centralProduccio(idPais, idLocalitat, idAdreca, energiaMax, dataInspeccio, estat, dataEstat)
		VALUES (PidPais, PidLocalitat, PidAdreca, PenergiaMax, campdataalta, 'A', campdataalta);
      
	-- Inserci a la taula de centrals nuclears
	INSERT INTO eolica(codiCentral, molins) VALUES
		(seq_central.currval, Pmolins);

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noMolins THEN
				resultat := 'ERROR: <Molins> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Central Elica ja est activa a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Eolica_insert;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO SOLAR - insert
-- Parmetres d'entrada: idPais, idLocalitat, idAdreca, energiaMax, panells
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Solar_insert (
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	Ppanells IN eolica.molins%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Solar_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	panells INTEGER;

  	noPais EXCEPTION;
	noLoc EXCEPTION;
  	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noPanells EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || Ppanells || '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada
	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Ppanells IS NULL OR Ppanells = 0) THEN
            RAISE noPanells;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Inserci a la taula de centrals de produccio (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO centralProduccio(idPais, idLocalitat, idAdreca, energiaMax, dataInspeccio, estat, dataEstat)
		VALUES (PidPais, PidLocalitat, PidAdreca, PenergiaMax, campdataalta, 'A', campdataalta);
      
	-- Inserci a la taula de centrals nuclears
	INSERT INTO solar(codiCentral, panells) VALUES
		(seq_central.currval, Ppanells);

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPanells THEN
				resultat := 'ERROR: <Panells> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Central Solar ja est activa a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Solar_insert;
/

CREATE OR REPLACE PROCEDURE p_CentralProduccio_canviEstat (
	PcodiCentral IN centralProduccio.codiCentral%TYPE, 
	Pestat IN centralProduccio.estat%TYPE, 
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_CentralProduccio_canviEstat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE;
	Pestat_actual CHAR(1);

	codiCentral INTEGER;

	noCodiCentral EXCEPTION;
	noTrobatCentral EXCEPTION;
	noEstat EXCEPTION;
  
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '> <' || Pestat || '>' ;
	campdatamodifica := SYSDATE;

	-- Comprovaci parmetres entrada
	IF (PcodiCentral IS NULL OR PcodiCentral = 0) THEN
            RAISE noCodiCentral;
	END IF;
	IF (Pestat <> 'A'  AND Pestat <> 'B') THEN
            RAISE noEstat;
	END IF;


	-- Comprovaci que existeixi la central
	BEGIN 
		SELECT estat INTO Pestat_actual FROM CentralProduccio
            WHERE codiCentral = PcodiCentral;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatCentral;
	END; 

	
	UPDATE centralProduccio SET
		estat = Pestat,
		dataEstat = campdatamodifica
	WHERE codiCentral = PcodiCentral;


	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <CodiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: Central no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEstat THEN
				resultat := 'ERROR: <Estat> erroni o inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);        
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_CentralProduccio_canviEstat;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO NUCLEAR - modifica
-- Parmetres d'entrada: codiCentral, idPais, idLocalitat, idAdreca, energiaMax,dataInspeccio, energiaMin,rebuig
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Nuclear_update (
	PcodiCentral IN centralProduccio.codiCentral%TYPE,
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	PdataInspeccio IN centralProduccio.dataInspeccio%TYPE,
	PenergiaMin IN nuclear.energiaMin%TYPE,
	Prebuig IN nuclear.rebuig%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Nuclear_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);
	Pestat_actual CHAR(1);
	estat CHAR(1);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	energiaMin NUMBER(10,2);
	rebuig NUMBER(10,2);
	rebuig_Actual NUMBER(10,2);

	noCodiCentral EXCEPTION;
  noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noEnergiaMin EXCEPTION;
	noRebuig EXCEPTION;
	noDataInspeccio EXCEPTION;

	noTrobatAdreca EXCEPTION;
	noTrobatCentral EXCEPTION;
	noTrobatNuclear EXCEPTION;
	CentralNoActiva EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PDataInspeccio || '> <' || PenergiaMin || '> <' || Prebuig || '>' ;


	-- Comprovaci parmetres entrada
	IF (PcodiCentral IS NULL OR PcodiCentral = '') THEN
            RAISE noCodiCentral;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (PenergiaMin IS NULL OR PenergiaMin = 0.0) THEN
            RAISE noEnergiaMin;
	END IF;

	IF (Prebuig IS NULL OR Prebuig = 0.0) THEN
            RAISE noRebuig;
	END IF;

	IF (PDataInspeccio IS NULL OR PDataInspeccio = '') THEN
            RAISE noDataInspeccio;
	END IF;

	-- Comprovaci que la central existeixi
	BEGIN 
		SELECT estat INTO Pestat_actual FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatCentral;
	END; 	

	-- Si la central existeix i est activa comprovem que sigui nuclear
	IF (Pestat_actual = 'A') THEN 
	-- Comprovaci que la central sigui nuclear
		BEGIN 
			SELECT rebuig INTO rebuig_actual FROM Nuclear
			WHERE codiCentral = PcodiCentral;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobatNuclear;
		END; 	
	ELSE
		RAISE CentralNoActiva;
	END IF;
			
		-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Updates de les taules

	UPDATE centralProduccio SET
		idPais = PidPais,
		idLocalitat = PidLocalitat,
		idAdreca = PidAdreca,
		energiaMax = PenergiaMax,
		dataInspeccio = PdataInspeccio
	WHERE codiCentral = PcodiCentral;
	
	UPDATE nuclear SET
		energiaMin=PenergiaMin,
		rebuig=Prebuig
	WHERE codiCentral = PcodiCentral;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

 
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <codiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);  
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEnergiaMin THEN
				resultat := 'ERROR: <EnergiaMin> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noRebuig THEN
				resultat := 'ERROR: <Rebuig> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataInspeccio THEN
				resultat := 'ERROR: <dataInspeccio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: Central no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatNuclear THEN
				resultat := 'ERROR: La central no s nuclear';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentralNoActiva THEN
				resultat := 'ERROR: La central no est activa. Cal activar-la per poder-ne actualitzar les dades';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);

			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Nuclear_update;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO HIDROELECTRICA - modifica
-- Parmetres d'entrada: codiCentral, idPais, idLocalitat, idAdreca, energiaMax,dataInspeccio, turbines
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Hidroelectrica_update (
	PcodiCentral IN centralProduccio.codiCentral%TYPE,
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	PdataInspeccio IN centralProduccio.dataInspeccio%TYPE,
	Pturbines IN hidroelectrica.turbines%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Hidroelectrica_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);
	Pestat_actual CHAR(1);
	estat CHAR(1);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	energiaMin NUMBER(10,2);
	turbines INTEGER;
	turbines_Actual INTEGER;

	noCodiCentral EXCEPTION;
  noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noTurbines EXCEPTION;
	noDataInspeccio EXCEPTION;

	noTrobatAdreca EXCEPTION;
	noTrobatCentral EXCEPTION;
	noTrobatHidroelectrica EXCEPTION;
	CentralNoActiva EXCEPTION;

	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PdataInspeccio || '> <' || Pturbines || '>' ;


	-- Comprovaci parmetres entrada
	IF (PcodiCentral IS NULL OR PcodiCentral = '') THEN
            RAISE noCodiCentral;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Pturbines IS NULL OR Pturbines = 0) THEN
            RAISE noTurbines;
	END IF;

	IF (PDataInspeccio IS NULL OR PDataInspeccio = '') THEN
            RAISE noDataInspeccio;
	END IF;

	-- Comprovaci que la central existeixi
	BEGIN 
		SELECT estat INTO Pestat_actual FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatCentral;
	END; 	

	-- Si la central existeix i est activa comprovem que sigui hidroelectrica
	IF (Pestat_actual = 'A') THEN 
	-- Comprovaci que la central sigui hidroelectrica
		BEGIN 
			SELECT turbines INTO turbines_actual FROM Hidroelectrica
			WHERE codiCentral = PcodiCentral;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobatHidroelectrica;
		END; 	
	ELSE
		RAISE CentralNoActiva;
	END IF;
			
		-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Updates de les taules

	UPDATE centralProduccio SET
		idPais = PidPais,
		idLocalitat = PidLocalitat,
		idAdreca = PidAdreca,
		energiaMax = PenergiaMax,
		dataInspeccio = PdataInspeccio
	WHERE codiCentral = PcodiCentral;
	
	UPDATE Hidroelectrica SET
		turbines=Pturbines
	WHERE codiCentral = PcodiCentral;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

 
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <codiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);  
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTurbines THEN
				resultat := 'ERROR: <Turbines> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataInspeccio THEN
				resultat := 'ERROR: <dataInspeccio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: Central no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatHidroelectrica THEN
				resultat := 'ERROR: La central no s hidroelectrica';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentralNoActiva THEN
				resultat := 'ERROR: La central no est activa. Cal activar-la per poder-ne actualitzar les dades';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Hidroelectrica_update;
/


------------------------------------------------------
-- CENTRAL PRODUCCIO SOLAR - modifica
-- Parmetres d'entrada: codiCentral, idPais, idLocalitat, idAdreca, energiaMax,dataInspeccio, panells
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Solar_update (
	PcodiCentral IN centralProduccio.codiCentral%TYPE,
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	PdataInspeccio IN centralProduccio.dataInspeccio%TYPE,
	Ppanells IN solar.panells%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Solar_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);
	Pestat_actual CHAR(1);
	estat CHAR(1);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	energiaMin NUMBER(10,2);
	panells INTEGER;
	panells_Actual INTEGER;

	noCodiCentral EXCEPTION;
  	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noPanells EXCEPTION;

	noTrobatAdreca EXCEPTION;
	noTrobatCentral EXCEPTION;
	noTrobatSolar EXCEPTION;
	CentralNoActiva EXCEPTION;
	noDataInspeccio EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PdataInspeccio || '> <' || Ppanells || '>' ;


	-- Comprovaci parmetres entrada
	IF (PcodiCentral IS NULL OR PcodiCentral = '') THEN
            RAISE noCodiCentral;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Ppanells IS NULL OR Ppanells = 0) THEN
            RAISE noPanells;
	END IF;

	IF (PDataInspeccio IS NULL OR PDataInspeccio = '') THEN
            RAISE noDataInspeccio;
	END IF;

	-- Comprovaci que la central existeixi
	BEGIN 
		SELECT estat INTO Pestat_actual FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatCentral;
	END; 	

	-- Si la central existeix i est activa comprovem que sigui solar
	IF (Pestat_actual = 'A') THEN 
	-- Comprovaci que la central sigui solar
		BEGIN 
			SELECT panells INTO panells_actual FROM solar
			WHERE codiCentral = PcodiCentral;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobatSolar;
		END; 	
	ELSE
		RAISE CentralNoActiva;
	END IF;
			
		-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Updates de les taules

	UPDATE centralProduccio SET
		idPais = PidPais,
		idLocalitat = PidLocalitat,
		idAdreca = PidAdreca,
		energiaMax = PenergiaMax,
		dataInspeccio = PdataInspeccio
	WHERE codiCentral = PcodiCentral;
	
	UPDATE Solar SET
		panells=Ppanells
	WHERE codiCentral = PcodiCentral;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

 
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <codiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPanells THEN
				resultat := 'ERROR: <Panells> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataInspeccio THEN
				resultat := 'ERROR: <dataInspeccio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: Central no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatSolar THEN
				resultat := 'ERROR: La central no s solar';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentralNoActiva THEN
				resultat := 'ERROR: La central no est activa. Cal activar-la per poder-ne actualitzar les dades';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Solar_update;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO EOLICA - modifica
-- Parmetres d'entrada: codiCentral, idPais, idLocalitat, idAdreca, energiaMax,dataInspeccio, molins
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Eolica_update (
	PcodiCentral IN centralProduccio.codiCentral%TYPE,
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	PdataInspeccio IN centralProduccio.dataInspeccio%TYPE,
	Pmolins IN eolica.molins%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Eolica_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);
	Pestat_actual CHAR(1);
	estat CHAR(1);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	energiaMin NUMBER(10,2);
	molins INTEGER;
	molins_Actual INTEGER;

	noCodiCentral EXCEPTION;
  	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noMolins EXCEPTION;

	noTrobatAdreca EXCEPTION;
	noTrobatCentral EXCEPTION;
	noTrobatEolica EXCEPTION;
	CentralNoActiva EXCEPTION;

	noDataInspeccio EXCEPTION;
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PdataInspeccio || '> <' || Pmolins || '>' ;


	-- Comprovaci parmetres entrada
	IF (PcodiCentral IS NULL OR PcodiCentral = '') THEN
            RAISE noCodiCentral;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Pmolins IS NULL OR Pmolins = 0) THEN
            RAISE noMolins;
	END IF;

	IF (PDataInspeccio IS NULL OR PDataInspeccio = '') THEN
            RAISE noDataInspeccio;
	END IF;

	-- Comprovaci que la central existeixi
	BEGIN 
		SELECT estat INTO Pestat_actual FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatCentral;
	END; 	

	-- Si la central existeix i est activa comprovem que sigui eolica
	IF (Pestat_actual = 'A') THEN 
	-- Comprovaci que la central sigui solar
		BEGIN 
			SELECT molins INTO molins_actual FROM eolica
			WHERE codiCentral = PcodiCentral;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobatEolica;
		END; 	
	ELSE
		RAISE CentralNoActiva;
	END IF;
			
		-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Updates de les taules

	UPDATE centralProduccio SET
		idPais = PidPais,
		idLocalitat = PidLocalitat,
		idAdreca = PidAdreca,
		energiaMax = PenergiaMax,
		dataInspeccio = PdataInspeccio
	WHERE codiCentral = PcodiCentral;
	
	UPDATE Eolica SET
		molins=Pmolins
	WHERE codiCentral = PcodiCentral;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

 
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <codiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noMolins THEN
				resultat := 'ERROR: <Molins> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataInspeccio THEN
				resultat := 'ERROR: <dataInspeccio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: Central no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatEolica THEN
				resultat := 'ERROR: La central no s eolica';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentralNoActiva THEN
				resultat := 'ERROR: La central no est activa. Cal activar-la per poder-ne actualitzar les dades';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Eolica_update;
/

------------------------------------------------------
-- CENTRAL PRODUCCIO Termica - modifica
-- Parmetres d'entrada: codiCentral, idPais, idLocalitat, idAdreca, energiaMax,dataInspeccio, emissions
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Termica_update (
	PcodiCentral IN centralProduccio.codiCentral%TYPE,
	PidPais IN centralProduccio.idPais%TYPE, 
	PidLocalitat IN centralProduccio.idLocalitat%TYPE,
	PidAdreca IN centralProduccio.idAdreca%TYPE,
	PenergiaMax IN centralProduccio.energiaMax%TYPE,
	PdataInspeccio IN centralProduccio.dataInspeccio%TYPE,
	Pemissions IN termica.emissions%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_Termica_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	nom VARCHAR2(50);
	Pestat_actual CHAR(1);
	estat CHAR(1);

	codiCentral INTEGER;
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	energiaMax NUMBER(10,2);
	emissions NUMBER(10,2);
	emissions_Actual NUMBER(10,2);

	noCodiCentral EXCEPTION;
  	noPais EXCEPTION;
	noLoc EXCEPTION;
	noAdreca EXCEPTION;
	noEnergiaMax EXCEPTION;
	noEmissions EXCEPTION;

	noTrobatAdreca EXCEPTION;
	noTrobatCentral EXCEPTION;
	noTrobatTermica EXCEPTION;
	CentralNoActiva EXCEPTION;

	noDataInspeccio EXCEPTION;
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '> <' || PidPais || '> <' || PidLocalitat || '> <' || PidAdreca || '> <' || PenergiaMax || 
			'> <' || PdataInspeccio || '> <' || Pemissions || '>' ;


	-- Comprovaci parmetres entrada
	IF (PcodiCentral IS NULL OR PcodiCentral = '') THEN
            RAISE noCodiCentral;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	IF (PenergiaMax IS NULL OR PenergiaMax = 0.0) THEN
            RAISE noEnergiaMax;
	END IF;

	IF (Pemissions IS NULL OR Pemissions = 0) THEN
            RAISE noEmissions;
	END IF;

	IF (PDataInspeccio IS NULL OR PDataInspeccio = '') THEN
            RAISE noDataInspeccio;
	END IF;

	-- Comprovaci que la central existeixi
	BEGIN 
		SELECT estat INTO Pestat_actual FROM CentralProduccio
		WHERE codiCentral = PcodiCentral;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatCentral;
	END; 	

	-- Si la central existeix i est activa comprovem que sigui eolica
	IF (Pestat_actual = 'A') THEN 
	-- Comprovaci que la central sigui solar
		BEGIN 
			SELECT emissions INTO emissions_actual FROM termica
			WHERE codiCentral = PcodiCentral;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobatTermica;
		END; 	
	ELSE
		RAISE CentralNoActiva;
	END IF;
			
		-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT estat INTO Pestat_actual FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatAdreca;
	END; 

	-- Updates de les taules

	UPDATE centralProduccio SET
		idPais = PidPais,
		idLocalitat = PidLocalitat,
		idAdreca = PidAdreca,
		energiaMax = PenergiaMax,
		dataInspeccio = PdataInspeccio
	WHERE codiCentral = PcodiCentral;
	
	UPDATE Termica SET
		emissions=Pemissions
	WHERE codiCentral = PcodiCentral;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);

 
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <codiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);			
      WHEN noEnergiaMax THEN
				resultat := 'ERROR: <EnergiaMax> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEmissions THEN
				resultat := 'ERROR: <Emissions> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataInspeccio THEN
				resultat := 'ERROR: <dataInspeccio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: Central no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatTermica THEN
				resultat := 'ERROR: La central no s Termica';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentralNoActiva THEN
				resultat := 'ERROR: La central no est activa. Cal activar-la per poder-ne actualitzar les dades';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Termica_update;
/


------------------------------------------------------
-- PRODUCCIO - insert
-- Parmetres d'entrada: codiCentral, dataLectura, ProduccioMes
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Produccio_insert (
  PcodiCentral IN produccio.codiCentral%TYPE,
  PdataLectura IN produccio.dataLectura%TYPE,
  Pproduccio IN produccio.produccioMes%TYPE,
  resultat OUT NOCOPY VARCHAR2
)
IS
	procediment VARCHAR2(80) := 'p_produccio_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE ;
  energ NUMBER(10,2) DEFAULT 0.;
  energMin NUMBER(10,2);  

	codiCentral INTEGER;
	estat CHAR(1);
	Pestat_actual CHAR(1);

	noCodiCentral EXCEPTION;
	nodataLectura EXCEPTION;
	noProduccio EXCEPTION;
	noTrobatCentral EXCEPTION;
  CentralNoActiva EXCEPTION;
  errorProduccio EXCEPTION;
	noEnergy EXCEPTION;
  noNuclear EXCEPTION;
  
  diff number(10,2);

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentral || '> <' || PdataLectura || '> <' || PProduccio || '>' ;
	campdatamodifica := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (PcodiCentral IS NULL OR PcodiCentral = 0) THEN
    RAISE nocodiCentral;
	END IF;

	IF (PdataLectura IS NULL OR PdataLectura = '') THEN
    RAISE nodataLectura;
	END IF;

	IF (PProduccio IS NULL OR PProduccio = 0.0) THEN
    RAISE noProduccio;
	END IF;

	-- Comprovaci que existeixi la central
	BEGIN 
		SELECT estat INTO Pestat_actual FROM centralProduccio
    WHERE PcodiCentral=codiCentral;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobatCentral;
	END; 

	SELECT energiaMax INTO energ FROM centralProduccio
    WHERE PcodiCentral=codiCentral;
            
	IF (Pproduccio > energ) THEN 
      	RAISE errorProduccio; 
	END IF;

-- obtenim l'energia minima de funcionament (=0. si la central no s nuclear)
  begin
    select energiaMin into energ from nuclear where codiCentral=PcodiCentral;
  exception
    when no_data_found then
      energ:=0.0;
  end;

  if ((Pproduccio-energ) > 0) then
    IF (Pestat_actual = 'A') THEN
	-- Insert
        INSERT INTO produccio(dataLectura, codiCentral, produccioMes) VALUES (PdataLectura, PcodiCentral, Pproduccio);
    ELSE 	
        RAISE CentralNoActiva;
    END IF;
  ELSE
      RAISE noEnergy;
  END IF;

 	resultat:='Ok';
  	p_e5_insert(PdataLectura,resultat);
  
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentral THEN
				resultat := 'ERROR: <codiCentral> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noProduccio THEN
				resultat := 'ERROR: <Produccio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataLectura THEN
				resultat := 'ERROR: <dataLectura> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentral THEN
				resultat := 'ERROR: Central no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentralNoActiva THEN
				resultat := 'ERROR: La central no est activa a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN errorProduccio THEN
				resultat := 'ERROR: La produccio introduda s superio a la mxima energia que pot produir la central';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEnergy THEN
				resultat := 'ERROR: No es supera energia minima funcionament';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Registre duplicat';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Produccio_insert;
/
