-- GESTIO CENTRALETES I LINIES COMUNICACIO (ALTA-BAIXA-MODIFICACIO) --

------------------------------------------------------
-- LINIA COMUNICACI - insert
-- Parmetres d'entrada: capacitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_LiniaComunicacio_insert (
	Pcapacitat IN LiniesComunicacio.capacitatLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_LiniaComunicacio_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;

	noCapacitat EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || Pcapacitat|| '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (Pcapacitat IS NULL OR Pcapacitat = 0.0) THEN
            RAISE noCapacitat;
	END IF;

	-- Inserci a la taula (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO LiniesComunicacio(capacitatLinia, estat, dataEstat)
		VALUES (Pcapacitat, 'A', campdataalta);
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCapacitat THEN
				resultat := 'ERROR: <CapacitatLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Linia ja est activa a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_LiniaComunicacio_insert;
/

------------------------------------------------------
-- LINIA COMUNICACI - canvi Estat
-- Parmetres d'entrada: idLinia
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_LiniaComunicacio_canviEstat (
	PidLinia IN LiniesComunicacio.idLinia%TYPE,
	Pestat IN LiniesComunicacio.estat%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_LiniaComunicacio_canviEstat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE;

	estat CHAR(1);
	estat_actual CHAR(1);
	
	noEstat EXCEPTION;
	noIdLinia EXCEPTION;
	noTrobatLinia EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidLinia || '> <' || Pestat || '>' ;
	campdatamodifica := SYSDATE;
	
	-- Comprovaci parmetres entrada

	IF (PidLinia IS NULL OR PidLinia = 0) THEN
            RAISE noidLinia;
	END IF;
	IF (Pestat <> 'A' AND Pestat <> 'B') THEN
            RAISE noEstat;
	END IF;


	-- Comprovaci existeix linia
	BEGIN
    SELECT estat INTO estat_actual FROM LiniesComunicacio
  		WHERE idLinia=PidLinia;
	EXCEPTION
    WHEN NO_DATA_FOUND THEN
      RAISE noTrobatLinia;
	END;

	UPDATE LiniesComunicacio SET
		estat = Pestat,
		dataEstat = campdatamodifica
	WHERE idLinia=PidLinia;			
	
       
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
		WHEN noidLinia THEN
			resultat := 'ERROR: <idLinia> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noEstat THEN
			resultat := 'ERROR: <estat> inexistent o erroni';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN noTrobatLinia THEN
			resultat := 'ERROR: No s ha trobat la Linia la BD';
			p_log_insert(procediment, entrada, sortida, resultat);
		WHEN OTHERS THEN
			err_num := SQLCODE;
			err_msg := SQLERRM;
			resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
			p_log_insert(procediment, entrada, sortida, resultat);
END p_LiniaComunicacio_canviEstat;
/

------------------------------------------------------
-- LINIA COMUNICACI - update
-- Parmetres d'entrada: idLinia, capacitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_LiniaComunicacio_update (
	PidLinia IN LiniesComunicacio.idLinia%TYPE,
	Pcapacitat IN LiniesComunicacio.capacitatLinia%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_LiniaComunicacio_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';

	estat CHAR(1);
	estat_actual CHAR(1);

	noCapacitat EXCEPTION;
	noidLinia EXCEPTION;
	noTrobatLinia EXCEPTION;
	LiniaNoActiva EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PidLinia || '> <' || Pcapacitat || '>' ;

	-- Comprovaci parmetres entrada

	IF (PidLinia IS NULL OR PidLinia = 0) THEN
            RAISE noidLinia;
	END IF;

	IF (Pcapacitat IS NULL OR Pcapacitat = 0.0) THEN
            RAISE noCapacitat;
	END IF;

	-- Comprovaci existeix linia
  BEGIN	
    SELECT estat INTO estat_actual FROM LiniesComunicacio
      WHERE idLinia=PidLinia;
	EXCEPTION
    WHEN NO_DATA_FOUND THEN
      RAISE noTrobatLinia;
	END;

	IF (estat_actual = 'A') THEN
		UPDATE LiniesComunicacio SET
			capacitatLinia = Pcapacitat
		WHERE idLinia=PidLinia;			
	ELSE
		RAISE LiniaNoActiva;
	END IF;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCapacitat THEN
				resultat := 'ERROR: <CapacitatLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noidLinia THEN
				resultat := 'ERROR: <idLinia> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLinia THEN
				resultat := 'ERROR: La linia no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN LiniaNoActiva THEN
				resultat := 'ERROR: La linia no est activa. Cal activar-la abans de modificar-la';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_LiniaComunicacio_update;
/


------------------------------------------------------
-- CENTRALETA DISTRIBUCI - insert
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca, capacitatCentraleta
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Centraleta_insert (
  PcodiCentraleta IN centraletadistribucio.codiCentraleta%TYPE,
	PidPais IN centraletaDistribucio.idPais%TYPE, 
	PidLocalitat IN centraletaDistribucio.idLocalitat%TYPE,
	PidAdreca IN centraletaDistribucio.idAdreca%TYPE,
	Pcapacitat IN centraletaDistribucio.capacitatcentraleta%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_centraleta_insert';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	codiCentraleta VARCHAR(10);
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	capacitatCentraleta NUMBER(10,2);

	nocodiCentraleta EXCEPTION;
	noCapacitat EXCEPTION;
  	noPais EXCEPTION;
	noLoc EXCEPTION;
  	noAdreca EXCEPTION;
	noTrobatAdreca EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' 
			|| PidAdreca || '> <' || Pcapacitat || '>' ;
	campdataalta := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (PcodiCentraleta IS NULL OR PcodiCentraleta = '') THEN
            RAISE nocodiCentraleta;
	END IF;

	IF (Pcapacitat IS NULL OR Pcapacitat = '') THEN
            RAISE noCapacitat;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE PidPais = idPais AND PidLocalitat=idLocalitat AND PidAdreca=idAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatAdreca;
	END; 

	-- Inserci a la taula (per defecte s'assigna com a estat 'A' = actiu a la BD)
	INSERT INTO centraletaDistribucio(codiCentraleta, idPais, idLocalitat, idAdreca, capacitatCentraleta, estat, dataEstat)
		VALUES (PcodiCentraleta, PidPais, PidLocalitat, PidAdreca, Pcapacitat, 'A', campdataalta);
        
	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <codiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCapacitat THEN
				resultat := 'ERROR: <Capacitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: No s ha trobat l adreca';
				p_log_insert(procediment, entrada, sortida, resultat);			
      WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Centraleta ja existeix a la BD';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Centraleta_insert;
/

------------------------------------------------------
-- CENTRALETA DISTRIBUCI - update
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca, capacitatCentraleta
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Centraleta_update(
      PcodiCentraleta IN centraletadistribucio.codiCentraleta%TYPE,
	PidPais IN centraletaDistribucio.idPais%TYPE, 
	PidLocalitat IN centraletaDistribucio.idLocalitat%TYPE,
	PidAdreca IN centraletaDistribucio.idAdreca%TYPE,
	Pcapacitat IN centraletaDistribucio.capacitatcentraleta%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_centraleta_update';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdataalta DATE;
	nom VARCHAR2(50);

	codiCentraleta VARCHAR(10);
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;
	capacitatCentraleta NUMBER(10,2);

	estat_actual CHAR(1);

	nocodiCentraleta EXCEPTION;
	noCapacitat EXCEPTION;
  	noPais EXCEPTION;
	noLoc EXCEPTION;
  	noAdreca EXCEPTION;
	noTrobatCentraleta EXCEPTION;
	noTrobatAdreca EXCEPTION;
	CentraletaNoActiva EXCEPTION;

	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' 
			|| PidAdreca || '> <' || Pcapacitat || '>' ;

	-- Comprovaci parmetres entrada

	IF (PcodiCentraleta IS NULL OR PcodiCentraleta = '') THEN
            RAISE nocodiCentraleta;
	END IF;

	IF (Pcapacitat IS NULL OR Pcapacitat = '') THEN
            RAISE noCapacitat;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE idPais = PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatAdreca;
	END; 

	-- Comprovacio que existeixi la centraleta
	BEGIN 
		SELECT estat INTO estat_actual FROM centraletaDistribucio
		WHERE codiCentraleta=PcodiCentraleta AND idPais = PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca;   
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatCentraleta;
	END; 

	IF (estat_actual = 'A') THEN
		UPDATE centraletaDistribucio SET
			capacitatCentraleta = Pcapacitat
		WHERE codiCentraleta=PcodiCentraleta AND idPais = PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca; 
	ELSE 
		RAISE CentraletaNoActiva;
	END IF;

	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <codiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCapacitat THEN
				resultat := 'ERROR: <Capacitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta THEN
				resultat := 'ERROR: La centraleta no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN CentraletaNoActiva THEN
				resultat := 'ERROR: La Centraleta no est activa a la BD. Per modificar cal activar-la';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Centraleta_update;
/


------------------------------------------------------
-- CENTRALETA DISTRIBUCI - canviEstat
-- Parmetres d'entrada: codiCentraleta, idPais, idLocalitat, idAdreca, estat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_Centraleta_canviEstat(
      PcodiCentraleta IN centraletadistribucio.codiCentraleta%TYPE,
	PidPais IN centraletaDistribucio.idPais%TYPE, 
	PidLocalitat IN centraletaDistribucio.idLocalitat%TYPE,
	PidAdreca IN centraletaDistribucio.idAdreca%TYPE,
	Pestat IN centraletaDistribucio.estat%TYPE,
	resultat OUT NOCOPY VARCHAR2
	)
IS
	procediment VARCHAR2(80) := 'p_centraleta_canviEstat';
	entrada VARCHAR2(4000) := ' ';
	sortida VARCHAR2(4000) := ' ';
	campdatamodifica DATE;
	nom VARCHAR2(50);

	codiCentraleta VARCHAR(10);
	idPais CHAR(2);
	idLocalitat INTEGER;
	idAdreca INTEGER;

	estat_actual CHAR(1);

	nocodiCentraleta EXCEPTION;
	noEstat EXCEPTION;
  	noPais EXCEPTION;
	noLoc EXCEPTION;
  	noAdreca EXCEPTION;
	noTrobatCentraleta EXCEPTION;
	noTrobatAdreca EXCEPTION;
	err_num NUMBER;
	err_msg VARCHAR2(255);
	
BEGIN
	-- Inicialitzaci de variables
	resultat := 'OK';
	entrada := '<' || PcodiCentraleta || '> <' || PidPais || '> <' || PidLocalitat || '> <' 
			|| PidAdreca || '> <' || Pestat || '>' ;
	campdatamodifica := SYSDATE;

	-- Comprovaci parmetres entrada

	IF (PcodiCentraleta IS NULL OR PcodiCentraleta = '') THEN
            RAISE nocodiCentraleta;
	END IF;

	IF (Pestat <> 'A' AND Pestat <> 'B') THEN
            RAISE noEstat;
	END IF;

	IF (PidPais IS NULL OR PidPais = '') THEN
            RAISE noPais;
	END IF;

	IF (PidLocalitat IS NULL OR PidLocalitat = 0) THEN
            RAISE noLoc;
	END IF;
        
	IF (PidAdreca IS NULL OR PidAdreca = 0) THEN
            RAISE noAdreca;
	END IF;

	-- Comprovaci que existeixi l'adrea
	BEGIN 
		SELECT carrer INTO nom FROM Adreca
            WHERE idPais = PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatAdreca;
	END; 

	-- Comprovacio que existeixi la centraleta
	BEGIN 
		SELECT estat INTO estat_actual FROM centraletaDistribucio
		WHERE codiCentraleta=PcodiCentraleta AND idPais = PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca;   
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			 RAISE noTrobatCentraleta;
	END; 

	UPDATE centraletaDistribucio SET
		estat = Pestat,
		dataEstat = campdatamodifica
	WHERE codiCentraleta=PcodiCentraleta AND idPais = PidPais AND idLocalitat=PidLocalitat AND idAdreca=PidAdreca; 


	-- Execuci correcta del procediment
	p_log_insert(procediment, entrada, sortida, resultat);
        
	-- Control d'excepcions
	EXCEPTION
			WHEN noCodiCentraleta THEN
				resultat := 'ERROR: <codiCentraleta> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEstat THEN
				resultat := 'ERROR: <Estat> inexistent o incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <idPais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLoc THEN
				resultat := 'ERROR: <idLocalitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAdreca THEN
				resultat := 'ERROR: <idAdreca> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatAdreca THEN
				resultat := 'ERROR: Adreca no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatCentraleta THEN
				resultat := 'ERROR: La centraleta no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_Centraleta_canviEstat;
/


